"""
Common
"""
from bee.db.psd.criteria import CriteriaSet
from bee.db.psd.result import Result, Reader, ExecuteResult
from bee.db.psd.table import Table
from bee.db.psd.value import Value


class ResultClause():
    name = "ResultClause"

    def result(self) -> Result:
        pass


"""
insert clauses
"""


class InsertClause():
    name = "InsertClause"

    def columns(self, columns=[]):
        pass


class InsertResultClause():
    name = "InsertResultClause"

    def submit(self):
        pass

    def result(self):
        pass


class InsertValuesClause(InsertResultClause):
    name = "InsertValuesClause"

    def values(self, values=[]) -> "InsertValuesClause":
        pass


class InsertColumnsClause():
    name = "InsertColumnsClause"

    def values(self, values=[]) -> InsertValuesClause:
        pass


"""
update clauses
"""


class UpdateClause():
    """
    UpdateClause
    """
    name = "UpdateClause"

    def set(self, col: str, val):
        pass

    def inc(self, col: str, val):
        pass

    def dec(self, col: str, val):
        pass

    def expr(self, col: str, val):
        pass


class SetClause(UpdateClause, ResultClause):
    """
    SetClause
    """
    name = "SetClause"

    def where(self, CriteriaSet) -> ResultClause:
        pass


"""
delete clauses
"""


class DeleteClause():
    """
    DeleteClause
    """
    name = "DeleteClause"

    def where(self, CriteriaSet) -> ResultClause:
        pass


"""
Select clauses
"""


class SelectClause():
    """
    SelectClause
    """
    name = "SelectClause"

    def From(self, table: Table) -> "FromClause":
        pass


class LimitClause():
    """
    LimitClause
    """
    name = "LimitClause"

    def limit(self, skip: int, take: int) -> "SelectResultClause":
        pass

    def page(self, index: int, size: int) -> "SelectResultClause":
        pass


class SelectResultClause():
    """
    SelectResultClause
    """
    name = "SelectResultClause"

    def value(self) -> Value:
        pass

    def int(self) -> int:
        pass

    def scan(self, *dst):
        pass

    # def fill(self, i: object):
    #     pass
    def one(self, _type: type):
        pass

    def list(self, _type: type):
        pass

    def reader(self) -> Reader:
        pass

    def for_(self, fn: object):
        pass


class OrderByClause(LimitClause, SelectResultClause):
    pass


class GroupByClause(LimitClause, SelectResultClause):
    def order_by(self, *orders) -> "OrderByClause":
        pass

    def having(self, f: CriteriaSet) -> "HavingClause":
        pass


class HavingClause(LimitClause, SelectResultClause):
    def order_by(self, *orders) -> "OrderByClause":
        pass


class WhereClause(LimitClause, SelectResultClause):

    def group_by(self, *cols) -> GroupByClause:
        pass

    def order_by(self, *orders) -> "OrderByClause":
        pass


class FromClause(LimitClause, SelectResultClause):
    """
    FromClause
    """
    name = "FromClause"

    def Join(self, table: Table, on: CriteriaSet) -> "JoinClause":
        pass

    def left_join(self, t: Table, on: CriteriaSet) -> "JoinClause":
        pass

    def right_join(self, t: Table, on: CriteriaSet) -> "JoinClause":
        pass

    def full_join(self, t: Table, on: CriteriaSet) -> "JoinClause":
        pass

    def Where(self, f: CriteriaSet) -> WhereClause:
        pass

class JoinClause(FromClause):
    pass

"""
count clauses
"""


class CountResultClause():
    """
    CountResultClause
    """
    name = "CountResultClause"

    def value(self) -> int:
        pass

    def scan(self, dst):
        pass


class CountClause(CountResultClause):
    """
    CountClause
    """
    name = "CountClause"

    def join(self, table: Table, on: CriteriaSet) -> JoinClause:
        pass

    def left_join(self, t: Table, on: CriteriaSet) -> JoinClause:
        pass

    def right_join(self, t: Table, on: CriteriaSet) -> JoinClause:
        pass

    def full_join(self, t: Table, on: CriteriaSet) -> JoinClause:
        pass

    def where(self, f: CriteriaSet) -> WhereClause:
        pass


class CountGroupByClause():
    def having(self, f: CriteriaSet) -> CountResultClause:
        pass


class CountWhereClause(CountResultClause):

    def group_by(self, *cols) -> "CountGroupByClause":
        pass


"""
execute clauses
"""


class ExecuteClause():

    def result(self) -> ExecuteResult:
        pass

    def value(self) -> Value:
        pass

    def scan(self, *dst):
        pass

    def fill(self, i: object):
        pass

    def reader(self) -> Reader:
        pass

    def for_(self, fn: object):
        pass



from enum import Enum
from typing import List

from bee.db.psd.column import Column


class CriteriaEnum(Enum):
    EQ = 0
    NE = 1
    LT = 2
    LTE = 3
    GT = 4
    GTE = 5
    IN = 6
    NIN = 7
    LK = 8


def pase_criteria_type(type_str: str) -> int:
    if type_str == "" or type_str == "eq":
        return CriteriaEnum.EQ
    elif type_str == "ne":
        return CriteriaEnum.NE
    elif type_str == "lt":
        return CriteriaEnum.LT
    elif type_str == "lte":
        return CriteriaEnum.LTE
    elif type_str == "gt":
        return CriteriaEnum.GT
    elif type_str == "gte":
        return CriteriaEnum.GTE
    elif type_str == "in":
        return CriteriaEnum.IN
    elif type_str == "nin":
        return CriteriaEnum.NIN
    elif type_str == "lk":
        return CriteriaEnum.LK
    else:
        raise TypeError("invalid criteria type: " + type_str)


class Criteria():
    """
    Criteria interface
    """
    pass


class ExprCriteria(str):
    pass


class OneColumnCriteria(Column):
    name = "OneColumnCriteria"

    def __init__(self, col, t: CriteriaEnum, val):
        self.col = col
        self.type = t
        self.value = val


class TwoColumnCriteria(Column):
    name = "TwoColumnCriteria"

    def __init__(self, left: Column, right: Column, t: CriteriaEnum):
        self.left = left
        self.right = right
        self.type = t


class CriteriaSet():
    name = "CriteriaSet"

    def empty(self) -> bool:
        pass


class JoinCriteriaSet(CriteriaSet):
    name = "JoinCriteriaSet"

    def left(self) -> CriteriaSet:
        pass

    def rght(self) -> CriteriaSet:
        pass

    def joiner(self) -> str:
        pass


class NotCriteriaSet(CriteriaSet):
    name = "NotCriteriaSet"

    def __init__(self, inner: CriteriaSet):
        self.inner = inner

    def empty(self) -> bool:
        self.inner.empty()
        return True
def not_(inner: CriteriaSet) -> CriteriaSet:
    return NotCriteriaSet(inner=inner)

class AndCriteriaSet(JoinCriteriaSet):
    name = "AndCriteriaSet"

    def __init__(self, left: CriteriaSet, right: CriteriaSet):
        self.left = left
        self.right = right

    def left(self) -> CriteriaSet:
        return self.left

    def rght(self) -> CriteriaSet:
        return self.right

    def empty(self) -> bool:
        return self.left.empty() and self.right.empty()

    def joiner(self) -> str:
        return "AND"


def and_(left: CriteriaSet, right: CriteriaSet) -> AndCriteriaSet:
    return AndCriteriaSet(left, right)


class OrCriteriaSet(JoinCriteriaSet):
    name = "OrCriteriaSet"

    def __init__(self, left: CriteriaSet, right: CriteriaSet):
        self.left = left
        self.right = right

    def left(self) -> CriteriaSet:
        return self.left

    def rght(self) -> CriteriaSet:
        return self.right

    def empty(self) -> bool:
        return self.left.empty() and self.right.empty()

    def joiner(self) -> str:
        return "OR"


def or_(left: CriteriaSet, right: CriteriaSet) -> OrCriteriaSet:
    return AndCriteriaSet(left, right)


class SimpleCriteriaSet(CriteriaSet):
    name = "SimpleCriteriaSet"

    def __init__(self):
        self.items = []

    def empty(self) -> bool:
        return self.items == None or len(self.items) == 0

    def add(self, col, t: int, val) -> "SimpleCriteriaSet":
        self.items.append(OneColumnCriteria(col, t, val))
        return self

    def add_if(self, when: bool, col, t: int, val) -> "SimpleCriteriaSet":
        if (when):
            self.add(col, t, val)
        return self

    def like(self, col, expr: str) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.LK, expr)

    def like_if(self, when, col, expr: str) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.LK, expr)

    def equal(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.EQ, val)

    def equal_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.EQ, val)

    def equal2(self, left: CriteriaSet, right: CriteriaSet) -> "SimpleCriteriaSet":
        return self.add2(left, right, CriteriaEnum.EQ)

    def not_equal(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.EQ, val)

    def not_equal_if(self, when, col, val):
        return self.add_if(when, col, CriteriaEnum.EQ, val)

    def not_equal2(self, left: CriteriaSet, right: CriteriaSet) -> "SimpleCriteriaSet":
        return self.add2(left, right, CriteriaEnum.NE)

    def add2(self, left, right, t: CriteriaEnum) -> "SimpleCriteriaSet":
        item = TwoColumnCriteria(left, right, t)
        self.items.append(item)
        return self

    def in_(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.IN, val)

    def in_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.IN, val)

    def not_in(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.NIN, val)

    def not_in_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.NIN, val)

    def less(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.LT, val)

    def less_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.LT, val)

    def less_or_equal(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.LTE, val)

    def less_or_equal_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.LTE, val)

    def less2(self, left: CriteriaSet, right: CriteriaSet) -> "SimpleCriteriaSet":
        return self.add(left, right, CriteriaEnum.LT)

    def less_or_equal2(self, left: CriteriaSet, right: CriteriaSet) -> "SimpleCriteriaSet":
        return self.add(left, right, CriteriaEnum.LTE)

    def greater(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.GT, val)

    def greater_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.GT, val)

    def greater_or_equal(self, col, val) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.GTE, val)

    def greater_or_equal_if(self, when, col, val) -> "SimpleCriteriaSet":
        return self.add_if(when, col, CriteriaEnum.GTE, val)

    def greater2(self, left: CriteriaSet, right: CriteriaSet) -> "SimpleCriteriaSet":
        return self.add(left, right, CriteriaEnum.GT)

    def greater_or_equal2(self, left: CriteriaSet, right: CriteriaSet) -> "SimpleCriteriaSet":
        return self.add(left, right, CriteriaEnum.GTE)

    def between(self, col, start, end) -> "SimpleCriteriaSet":
        return self.add(col, CriteriaEnum.GTE, start).add(col, CriteriaEnum.LTE, end)

    def prefix(self, col, s) -> "SimpleCriteriaSet":
        return self.like(col, s + "%")

    def suffix(self, col, s) -> "SimpleCriteriaSet":
        return self.like(col, "%" + s)

    def contains(self, col, s) -> "SimpleCriteriaSet":
        return self.like(col, "%" + s + "%")

    def date(self, col, date) -> "SimpleCriteriaSet":
        pass

    def date_range(self, col, start, end) -> "SimpleCriteriaSet":
        pass

def equal(col, val) -> "SimpleCriteriaSet":
    set = SimpleCriteriaSet()
    return set.add(col, CriteriaEnum.EQ, val)


# Ported from https://github.com/graham/python_xid which almost a direct copy of xid(https://github.com/rs/xid)

# implementation of RFC 4648 using extended hex alphabet
# https://tools.ietf.org/html/rfc4648#page-10
# Used by golang in a number of places but doesn't seem to
# exist in the python standard library.

from bee.data import const

const.guid_encode_char = "0123456789abcdefghijklmnopqrstuv"

decodeHexMap = {}
for index, i in enumerate(const.guid_encode_char):
    decodeHexMap[i] = index

padChar = "="


class BadPadding(Exception):
    pass


def encode(src, str_map):
    dst = []
    src_len = 0

    if len(src) == 0:
        return ''

    while len(src):
        src_len = len(src)
        next_byte = [0] * 8

        if src_len > 4:
            next_byte[7] = src[4] & 0x1f
            next_byte[6] = src[4] >> 5

        if src_len > 3:
            next_byte[6] = next_byte[6] | (src[3] << 3) & 0x1f
            next_byte[5] = (src[3] >> 2) & 0x1f
            next_byte[4] = src[3] >> 7

        if src_len > 2:
            next_byte[4] = next_byte[4] | (src[2] << 1) & 0x1f
            next_byte[3] = (src[2] >> 4) & 0x1f

        if src_len > 1:
            next_byte[3] = next_byte[3] | (src[1] << 4) & 0x1f
            next_byte[2] = (src[1] >> 1) & 0x1f
            next_byte[1] = (src[1] >> 6) & 0x1f

        if src_len > 0:
            next_byte[1] = next_byte[1] | (src[0] << 2) & 0x1f
            next_byte[0] = src[0] >> 3

        for nb in next_byte:
            dst.append(str_map[nb])

        src = src[5:]

    if src_len < 5:
        dst[-1] = '='
    if src_len < 4:
        dst[-2] = '='
        dst[-3] = '='
    if src_len < 3:
        dst[-4] = '='
    if src_len < 2:
        dst[-5] = '='
        dst[-6] = '='

    return ''.join(dst)


def decode(src, str_map):
    src = src.upper()

    end = False
    result = []
    while len(src) > 0 and not end:
        dst = [0] * 5
        dbuf = [0] * 8

        src_len = 8

        for i in range(0, 8):
            if i >= len(src):
                src_len = i
                end = True
                break
            char = src[i]
            if char == padChar:
                end = True
                src_len = i
                break
            else:
                dbuf[i] = decodeHexMap[char]

        if src_len >= 8:
            dst[4] = (dbuf[6] << 5) | (dbuf[7])
        if src_len >= 7:
            dst[3] = (dbuf[4] << 7) | (dbuf[5] << 2) | (dbuf[6] >> 3)
        if src_len >= 5:
            dst[2] = (dbuf[3] << 4) | (dbuf[4] >> 1)
        if src_len >= 4:
            dst[1] = (dbuf[1] << 6) | (dbuf[2] << 1) | (dbuf[3] >> 4)
        if src_len >= 2:
            dst[0] = (dbuf[0] << 3) | (dbuf[1] >> 2)

        dst = list(map(lambda x: x & 0xff, dst))

        if src_len == 2:
            dst = dst[:1]
        elif src_len == 4:
            dst = dst[:2]
        elif src_len == 5:
            dst = dst[:3]
        elif src_len == 7:
            dst = dst[:4]
        elif src_len == 8:
            dst = dst[:5]

        result.extend(dst)
        src = src[8:]

    return result


def b32encode(src):
    return encode(list(map(ord, src)), const.guid_encode_char)


def b32decode(src):
    return decode(src, const.guid_encode_char)



import contextlib
import types
from queue import Queue

from bee.errors.error import BeeError


class ObjectPool(object):

    def __init__(self, fn_cls, *args, **opts):
        super(ObjectPool, self).__init__()
        self.fn_cls = fn_cls
        self._myinit(*args, **opts)

    def _myinit(self, *args, **kwargs):
        self.queue = Queue()
        self.args = args
        self.min_pool_size = int(kwargs.get("min_pool_size", 0))
        self.max_pool_size = int(kwargs.get("max_pool_size", 5))
        while self.queue.qsize() < self.min_pool_size:
            self.queue.put(self._new())

    def _new(self):
        if type(self.fn_cls) == types.FunctionType:
            if len(self.args) > 0:
                return self.fn_cls(self.args)
            else:
                return self.fn_cls()
        elif type(self.fn_cls) == type:
            return self.fn_cls()
        else:
            raise BeeError("Wrong type")

    @contextlib.contextmanager
    def new(self):
        obj = self.borrow()
        try:
            yield obj
        except BeeError as e:
            yield None
        finally:
            self.recycle(obj)

    def borrow(self):
        if self.queue.qsize() < self.max_pool_size and self.queue.empty():
            self.queue.put(self._new())
        return self.queue.get()

    def recycle(self, obj):
        self.queue.put(obj)



from bee.db.psd.column import new_columns, ColumnFilter, Columns
from bee.db.psd.criteria import SimpleCriteriaSet, CriteriaSet, not_, and_, or_, equal
from bee.db.psd.select import Order, OrderEnum
from bee.db.psd.table import Table, new_table


def ASC(cols: str) -> Order:
    return Order(cols=C(cols), t=OrderEnum.ASC)


def DESC(*cols: str) -> Order:
    return Order(cols=C(cols), t=OrderEnum.DESC)


def T(name: str, *alias: str) -> Table:
    return new_table(name, *alias)


def C(*cols: str)  -> Columns:
    return new_columns(*cols)


def CT(t, *cols: str):
    return new_columns().get_table(t, *cols)


def CX(expr: str, alias: str=None):
    return new_columns().get_expr(expr, alias)


def Omit(*cols: str) -> ColumnFilter:
    # TODO: omit-filter
    pass


def Include(*cols: str) -> ColumnFilter:
    # TODO: include-filter
    pass


def W() -> SimpleCriteriaSet:
    return SimpleCriteriaSet()


def Not(inner: CriteriaSet) -> CriteriaSet:
    return not_(inner)


def And(left: CriteriaSet, right: CriteriaSet) -> CriteriaSet:
    return and_(left, right)


def Or(left: CriteriaSet, right: CriteriaSet) -> CriteriaSet:
    return or_(left, right)


def Equal(col, val) -> SimpleCriteriaSet:
    return equal(col, val)


def On(left, right) -> SimpleCriteriaSet:
    return W().equal2(left, right)



from typing import List

from bee.config.manager import Manager

m = Manager.new("app")

def find_file(name: str, *exts) -> str :
    return m.find_file(name, exts)

def find_files(name: str, *exts) -> List[str]:
    return m.find_files(name, exts)

def find_folder(name: str) -> str:
    return m.find_folder(name)

def find_folders(name: str) -> List[str]:
    return m.find_folders(name)

def set_env_prefix(prefix: str):
    m.set_env_prefix(prefix)

def set_profiles(*profiles: str):
    m.set_profiles(profiles)

def bind_env(key: str, env_key: str):
    m.bind_env(key, env_key)

def set_name(name: str):
    m.set_name(name)

def add_folder(*dirs: str):
    m.add_folder(dirs)

def add_source(*srcs):
    m.add_source(srcs)

def add_byte_source(d: bytes, t: str):
    m.add_byte_source(d, t)

def set_default_value(name: str, value):
    m.set_default_value(name, value)

def load():
    m.load(True)

def exist(key: str) -> bool:
    return m.get(key) != None

def get(key: str) -> object:
    return m.get(key)


import pickle

from typing import List

from bee.data.option import Options, Option
from bee.data.map import Map
from bee.errors.error import CodedError
from bee.net.rpc.codec import Stream, IClientCodec, IServerCodec, CodecBuilder, Request as _Request, RequestHead as _RequestHead, ResponseHead as _ResponseHead, \
    Response as _Response, Result as _Result, register_codec
from bee.data import const

# 缺省最大值
from bee.net.rpc.codecs.proto.msg_pb2 import Label, Response, Request

const.default_max_message_size = 2 << 20


def reset_req(req: Request):
    if req.args != None:
        req.args = req.args[:0]
    if req.labels != None:
        req.labels = req.labels[:0]


def reset_resp(resp: Response):
    resp.result = None
    resp.error = None


class ClientCodec(IClientCodec):
    """
    客户端编解码
    """

    def __init__(self, s: Stream, req: Request = None, resp: Response = None, max_msg_size=0):
        self.s = s
        self.req = req
        self.resp = resp
        self.max_msg_size = max_msg_size

    def stream(self) -> Stream:
        return self.s

    def encode(self, req: _Request):
        self.req.id = req.head.id
        self.req.service = req.head.service
        self.req.method = req.head.method

        if len(req.args) > 0:
            for v in req.args:
                # convert v to bytes, v is object
                self.req.args.append(pickle.dumps(v))

        if len(req.head.labels) > 0:
            for v in req.head.labels:
                self.req.labels.append(Label(name=v.name, value=v.value))

        self._write(self.req.SerializeToString())

    def _write(self, data: bytes):
        # wtire data's length, occupy 4 bytes
        self.s.write(len(data).to_bytes(4, byteorder='little'))
        # write data
        self.s.write(data)
        self.s.flush()

    def decode_head(self, head: _ResponseHead):
        length_bytes = self.s.read(4);
        length = int.from_bytes(length_bytes, byteorder="little")
        if length > self.max_msg_size:
            raise BaseException("message too big: %s" % length)

        data = self.s.read(length)
        self.resp.ParseFromString(data)
        head.id = self.resp.id

    def decode_result(self, result: _Result):
        if self.resp.result != None:
            result.error = None
            result.value = pickle.loads(self.resp.result)
        else:
            result.error = CodedError(self.resp.error)

    def discard_result(self):
        return None


class ServerCodec(IServerCodec):
    """
    服务端编解码
    """

    def __init__(self, s: Stream, req: Request = None, resp: Response = None, max_msg_size=0):
        self.s = s
        self.req = req
        self.resp = resp
        self.max_msg_size = max_msg_size

    def stream(self) -> Stream:
        return self.s

    def encode(self, resp: _Response):
        self.resp.id = resp.head.id

        if resp.result.error == None:
            if resp.result.value != None:
                self.resp.result = pickle.dumps(resp.result.value)
        else:
            self.resp.result = None
            self.resp.error = resp.result.error

        self._write(self.resp.SerializeToString())

    def _write(self, data: bytes):
        # wtire data's length, occupy 4 bytes
        self.s.write(len(data).to_bytes(4, byteorder='little'))
        # write data
        self.s.write(data)
        self.s.flush()

    def decode_head(self, head: _RequestHead):
        length_bytes = self.s.read(4);
        length = int.from_bytes(length_bytes, byteorder="little")
        if length > self.max_msg_size:
            raise BaseException("message too big: %s" % length)

        data = self.s.read(length)
        self.req.ParseFromString(data)
        head.id = self.req.id
        head.service = self.req.service
        head.method = self.req.method
        if self.req.labels:
            opts = Options()
            for v in self.req.labels:
                opts.append(Option(name=v.name, value=v.value))
            head.labels = opts
        if head.id != None and head.id > 0:
            print("%s, %s.%s(%s), labels(%s)" % (head.id, head.service, head.method, "", head.labels))

    def decode_args(self, args: List[object]):
        if self.req.args != None:
            for i, v in enumerate(self.req.args):
                args.insert(i, pickle.loads(v))

    def discard_args(self):
        return None


class Builder(CodecBuilder):

    name="proto"

    def new_client_codec(self, s: Stream, opts: Map) -> ClientCodec:
        max_msg_size = self.max_msg_size(opts)
        return ClientCodec(s=s, req=Request(), resp=Response(), max_msg_size=max_msg_size)

    def new_server_codec(self, s: Stream, opts: Map) -> ServerCodec:
        max_msg_size = self.max_msg_size(opts)
        return ServerCodec(s=s, req=Request(), resp=Response(), max_msg_size=max_msg_size)

    def max_msg_size(self, opts: Map):
        size = None
        if opts != None:
            if opts.get("max_msg_size") != None:
                size = int(opts.get("max_msg_size"))

        if size == None or size == 0:
            size = const.default_max_message_size

        return size


def init():
    register_codec("proto", Builder())



from typing import List

import redis
from redis import Redis
from redis.sentinel import Sentinel
from rediscluster import RedisCluster
from gevent.lock import BoundedSemaphore

from bee.data.map import Map
from bee import config

TYPE_SINGLE = "single"
TYPE_SENTINEL = "sentinel"
TYPE_CLUSTER = "cluster"


class Options(Map):

    def __init__(self, _type: str = None, address: List[str] = [], password: str = None
                 , socket_connect_timeout: int = 5000, socket_timeout: int = 5000, pool_size: int = None
                 , master: str = None, db: int = 0):
        self.type = _type
        self.address = address
        self.password = password
        self.socket_connect_timeout = socket_connect_timeout
        self.socket_timeout = socket_timeout
        self.pool_size = pool_size
        self.master = master
        self.db = db


class Client():

    def __init__(self, r: object = None, s: Sentinel = None, rc: RedisCluster = None, opts: Options = None):
        self._redis = r
        self._sentinel = s
        self._rc = rc
        self._opts = opts

    def writer(self) -> Redis:
        if self._opts.type == TYPE_SINGLE:
            return self._redis
        elif self._opts.type == TYPE_SENTINEL:
            return self._sentinel.master_for(self._opts.master, password=self._opts.password, db=self._opts.db
                                             , socket_timeout=self._opts.socket_timeout / 1000, socket_connect_timeout=self._opts.socket_connect_timeout / 1000)
        elif self._opts.type == TYPE_CLUSTER:
            return self._rc
        else:
            return None

    def reader(self) -> Redis:
        if self._opts.type == TYPE_SINGLE:
            return self._redis
        elif self._opts.type == TYPE_SENTINEL:
            return self._sentinel.slave_for(self._opts.master, password=self._opts.password, db=self._opts.db
                                            , socket_timeout=self._opts.socket_timeout / 1000, socket_connect_timeout=self._opts.socket_connect_timeout / 1000)
        elif self._opts.type == TYPE_CLUSTER:
            return self._rc
        else:
            return None


class Factory():

    def __init__(self):
        self.sem = BoundedSemaphore(1)
        self.cmds = Map()

    def open(self, name: str) -> Client:
        self.sem.acquire()
        client = self.cmds.get(name)
        if client == None:
            client = self.create(name)
        self.sem.release()
        return client

    def create(self, name: str) -> Client:
        client = self.cmds.get(name)
        if client != None:
            return client
        else:
            opts = self.load_options(name)
            if opts == None:
                return None

            if opts.type == TYPE_SINGLE:
                client = self.create_single(opts)
            elif opts.type == TYPE_SENTINEL:
                client = self.create_sentinel(opts)
            elif opts.type == TYPE_CLUSTER:
                client = self.create_cluster(opts)
            else:
                client = self.create_single(opts)
            self.cmds.set(name, client)
            return client

    def load_options(self, name) -> Options:
        key = "bee.data.redis." + name
        if not config.exist(key):
            pass

        opts = Options()

        bee_data_redis_conf = config.get(key)
        opts.cover(bee_data_redis_conf)
        return opts

    def create_single(self, opts: Options) -> Client:

        pair = opts.address[0].split(":")
        pool = redis.ConnectionPool(host=pair[0], port=int(pair[1]), max_connections=opts.pool_size, db=opts.db)
        r = redis.Redis(connection_pool=pool, socket_timeout=opts.socket_timeout / 1000,
                        socket_connect_timeout=opts.socket_connect_timeout / 1000)
        return Client(r=r, opts=opts)

    def create_sentinel(self, opts: Options) -> Client:

        address_array = []
        for x in opts.address:
            pair = x.split(":")
            address_array.append(tuple(pair))

        sentinel = Sentinel(address_array, socket_timeout=5)
        return Client(s=sentinel, opts=opts)

    def create_cluster(self, opts: Options) -> Client:
        startup_nodes = []
        for v in opts.address:
            pair = v.split(":")
            startup_nodes.append({"host": str(pair[0]), "port": pair[1]})
        rc = RedisCluster(startup_nodes=startup_nodes, decode_responses=True, password=opts.password
                          , max_connections=opts.pool_size, max_connections_per_node=20
                          , socket_timeout=opts.socket_timeout / 1000,
                        socket_connect_timeout=opts.socket_connect_timeout / 1000)
        return Client(rc=rc, opts=opts)



from typing import List

import yaml

from bee import config
from bee.data.map import Map
from bee.ext import files


class ITranslator():

    def get(self, key: str):
        pass

    def format(self, key: str, *args: str) -> str:
        pass

class Translator():

    def __init__(self, lang: str, m: Map):
        self.lang = lang
        self._m = m

    def get(self, key: str):
        return self._m.find(key)

    def format(self, key: str, *args: str) -> str:
        return str(self._m.get(key)).format(*args)


def new_Translator(lang: str, file_path) -> ITranslator:
    with open(file_path, "rb") as f:
        last_dot_index = file_path.rfind(".")
        t = file_path[last_dot_index + 1:]
        b = f.read()
        content = yaml.load(b, Loader=yaml.FullLoader)
        m = Map.from_dict(d=content)
        return Translator(lang=lang, m=m)


class Manager():

    def __init__(self, *dirs: str):
        self.dirs: List[str] = list(dirs)

    def find(self, lang: str):
        t = self.get(lang)
        if t != None:
            return t

        if lang.find("-") > -1:
            _lang = lang.split("-")[0]
            return self.get(_lang)

        return None

    def get(self, lang: str) -> ITranslator:
        dirs = self.get_dirs()
        for dir in dirs:
            file_name = dir + ".yml"
            if files.exist(file_name):
                with open(file_name, "rb") as f:
                    last_dot_index = file_name.rfind(".")
                    t = file_name[last_dot_index + 1:]
                    b = f.read()
                    content = yaml.load(b, Loader=yaml.FullLoader)
                    m = Map.from_dict(d=content)
                    return Translator(lang=lang, m=m)
            return None
        return None

    def all(self) -> Map[str, ITranslator]:
        tm = Map[str, ITranslator]()
        dirs = self.dirs
        for dir in dirs:
            matches = files.all_file(dir, "*.yml")
            if len(matches) > 0:
                for v in matches:
                    file_name = files.file_name(v)
                    lang = file_name.strip(".yml")
                    t = new_Translator(lang=lang, file_path=v)
                    tm.set(lang, t)

        return tm

    def get_dirs(self) -> List[str]:
        if len(self.dirs) == 0:
            self.dirs = config.find_folders("i18n")
        return self.dirs


def new(*dirs: str) -> "Manager":
    return Manager(*dirs)



from gevent import monkey;monkey.patch_all()
import gevent
from gevent import socket
from enum import Enum

from bee.errors.error import CodedError

from bee.data.guid.guid import Guid
from bee.data.map import Map
from bee.net.rpc.channel import Channel
from bee.net.rpc.codec import CodecBuilder, RequestHead, Request, ResponseHead, Result

from gsocketpool import Pool
from gsocketpool import TcpConnection
"""default max connectinos"""
default_max_connections = 100
"""default max error count"""
default_max_error_count = 10

class NodeState(Enum):
    """
    fail mode
    """
    """Idle indicates the Node is idle"""
    Idle = 0
    """Ready indicates the Node is ready for work"""
    Ready = 1
    """Shutdown indicates the Node has started shutting down"""
    Shutdown = 2


class Node():

    def __init__(self, id: str, address: str, status: NodeState = NodeState.Ready, cb: CodecBuilder = None,
                 connect_timeout=5):
        self.id = id
        self.address = address
        self.status = status
        self._cb = cb
        pair = address.split(":")
        options = dict(host=pair[0], port=int(pair[1]), timeout=connect_timeout)
        self.pool: Pool = Pool(TcpConnection, options,
                               # initial_connections=1,
                               max_connections=default_max_connections,
                               reap_expired_connections=False)
        self.error_count = 0

        self.count = 1

        # self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # self.sock.connect((pair[0], int(pair[1])))

    def __str__(self):
        return "Node(id={}, address={}, status={}(value={}))".format(self.id, self.address, self.status,
                                                                     self.status.value)
    def call(self, service: str, method: str, args=[]) -> Result:

        """
        class remote method
        :param service:
        :param method:
        :param args:
        :return:
        """
        print("Node.call(service={}, method={}, args={})".format(service, method, str(args)))
        try:
            with self.pool.connection() as client:
                if not client.is_connected():
                    client.open()
                sock = client.socket
                rh = RequestHead()
                rh.id = self.count
                self.count += 1
                rh.service = service
                rh.method = method
                rh.labels = []
                r = Request(head=rh, args=args)
                cc = self._cb.new_client_codec(s=Channel(id=Guid().string(), sock=sock), opts=Map())
                cc.encode(req=r)
                rh = ResponseHead()
                cc.decode_head(rh)
                rt = Result()
                cc.decode_result(rt)
                return rt

        except Exception as e:
            self.error_count +=1
            if (self.error_count >= default_max_error_count):
                self.report_error()
            raise CodedError(code=-1, message=e.__str__(), detail=e.__str__())

    def report_error(self):
        self.status = NodeState.Shutdown
        gevent.spawn(self.hearbeat())


    def hearbeat(self):
        #TODO; heartbeat retry
        pass


# -*- coding: utf-8 -*-
from bee.errors.error import ConstError


class _const:

    def __setattr__(self, key, value):
        if self.__dict__.__contains__(key):
            raise ConstError("can't rebind const (%s)" % key)
        self.__dict__[key] = value


import sys

sys.modules[__name__] = _const()



from typing import List

from bee.data.option import Options
from bee.data.map import Map
from bee.data import const

const.codecs = Map()


class ReadPeeker():
    def read(self, p: bytes) -> int:
        pass

    def peek(self, n: int) -> bytes:
        pass


class Stream(ReadPeeker):

    def id(self) -> str:
        pass

    def reader(self):
        pass

    def peek(self, n: int) -> bytes:
        pass

    def read(self, n: int) -> bytes:
        pass

    def read_bytes(self, separator=b'\n') -> bytes:
        pass

    def read_str(self, separator=b'\n') -> str:
        pass

    def writer(self):
        pass

    def write(self, p: bytes) -> int:
        pass

    def write_str(self, s:str):
        pass

    def flush(self):
        pass


class RequestHead():
    def __init__(self,id: int = None, service: str = None, method: str = None, labels: Options = None, assets: Options = None):
        self.id = id
        self.service = service
        self.method = method
        self.labels = labels
        self.assets = assets

class Request():

    def __init__(self, head: RequestHead = None, args: List[object] = []):
        self.head = head
        self.args = args


class ResponseHead():

    def __init__(self, id: int = None, assets: Options = None):
        self.id = id
        self.assets = assets



class Result():

    def __init__(self, value: object = None, error = None):
        self.value = value
        self.error = error


class Response():

    def __init__(self, head: ResponseHead = None, result: Result = None):
        self.head = head
        self.result = result


class IClientCodec():

    def stream(self) -> Stream:
        pass

    def encode(self, req: Request):
        pass

    def decode_head(self, head: ResponseHead):
        pass

    def decode_result(self, result: Result):
        pass

    def discard_result(self):
        pass


class IServerCodec():

    def stream(self) -> Stream:
        pass

    def encode(self, resp: Response):
        pass

    def decode_head(self, head: RequestHead):
        pass

    def decode_args(self, args: []):
        pass

    def discard_args(self):
        pass


class CodecBuilder():

    def new_client_codec(self, s: Stream, opts: Map) -> IClientCodec:
        pass

    def new_server_codec(self, s: Stream, opts: Map) -> IServerCodec:
        pass


def register_codec(name: str, cb: CodecBuilder):
    const.codecs.set(name, cb)

def get_codec(name: str) -> CodecBuilder:
    return const.codecs.get(name)


class Macther():

    def match(self, ReadPeeker) -> bool:
        pass

class Any(Macther):
    def match(self, ReadPeeker) -> bool:
        return True


import asyncio

from aiohttp import web

async def index(request):
    print(request.content)
    await asyncio.sleep(0.5)
    return web.Response(body=b'<h1>Index</h1>')

async def hello(request):
    await asyncio.sleep(0.5)
    text = '<h1>hello, %s!</h1>' % request.match_info['name']
    return web.Response(body=text.encode('utf-8'))

async def init(loop):
    app = web.Application(loop=loop)
    app.router.add_route('GET', '/', index)
    app.router.add_route('GET', '/hello/{name}', hello)
    srv = await loop.create_server(app.make_handler(), '127.0.0.1', 8000)
    print('Server started at http://127.0.0.1:8000...')
    return srv

loop = asyncio.get_event_loop()
loop.run_until_complete(init(loop))
loop.run_forever()


import contextlib
import logging
from datetime import datetime

from bee.data.map import Map
from bee.db.psd.clause import SelectClause
from bee.db.psd.column import new_columns, ColumnFilter, Columns
from bee.db.psd.connection.connection import IConnection
from bee.db.psd.connection.mysql import MysqlConnection
from bee.db.psd.delete import DeleteContext, DeleteInfo
from bee.db.psd.insert import InsertContext, InsertInfo
from bee.db.psd.select import SelectInfo, SelectContext
from bee.db.psd.update import UpdateContext, UpdateInfo
from bee.errors.error import BeeError

logging.basicConfig(level=logging.INFO)


class DBOptions(Map):

    def __init__(self, name: str = None, provider: str = "mysql"
                 # , address: str=""
                 , max_open_conns: int = 100, max_idle_conns: int = 5
                 , trace=Map(), options=Map()):
        self.name = name
        self.provider = provider
        # self.address = address
        self.max_open_conns = max_open_conns
        self.max_idle_conns = max_idle_conns
        # self.conn_lifetime = conn_lifetime,
        self.trace = trace
        self.options = options
        self.host = "127.0.0.1"
        self.port = 3306
        self.database = "test"
        self.user = "root"
        self.password = "000000"
        self.charset = "utf8mb4"


class DB():
    """
    database's CURD include both simple-sql(Insert,Update,Delete,Select) and orm(Create,Modify,Remove) styles.
    """

    # name = "DB"

    def insert(self, table: str):
        pass

    def update(self, table: str):
        pass

    def delete(self, table: str):
        pass

    def Select(self, *cols: str):
        pass

    def save(self, model, *filters: ColumnFilter):
        pass

    def modify(self, model, *filters: ColumnFilter):
        pass

    def remove(self, model):
        pass

    def Query(self, cols: Columns, *distinct):
        pass


def trace(func):
    def wrapper(*args, **kw):
        self = args[0]
        enabled = False
        if self.opts.trace["enabled"] != None and self.opts.trace["enabled"] == True:
            enabled = True

        begin = datetime.now().timestamp()
        if enabled:
            logging.info('psd > call %s()' % func.__name__)
        result = func(*args, **kw)
        end = datetime.now().timestamp()
        if enabled:
            logging.info("psd > sql: %s, args: %s(%s), time: %s", args[1], args[2], str(len(args[2])), str(end - begin))
        return result

    return wrapper


class Database(DB):
    name = "Database"

    def __init__(self, name="default", opts=DBOptions(), conn=IConnection(), provider=None, stmts={}):
        # def __init__(self, name="default", opts=Options(), conn=None):
        self.name = name
        self.opts = opts
        self.conn = conn
        self.p = provider
        self.stmts = stmts

    def name(self):
        return self.opts.name

    @trace
    def exec(self, query, args=[]):
        return self.conn.exec(query, args)

    @trace
    def find_one(self, query, args=[]):
        return self.conn.find_one(query, args)

    @trace
    def find_list(self, query, args=[]):
        return self.conn.find_list(query, args)

    def insert(self, table) -> InsertContext:
        info = InsertInfo(table=table)
        context = InsertContext(info=info, db=self)
        context.db = self
        context.insert(table)
        return context

    def update(self, table) -> UpdateContext:
        info = UpdateInfo(table=table)
        context = UpdateContext(info=info, db=self)
        context.db = self
        context.update(table)
        return context

    def delete(self, table) -> DeleteContext:
        info = DeleteInfo(table=table)
        context = DeleteContext(info=info, db=self)
        context.db = self
        context.delete(table)
        return context

    def Select(self, *cols) -> SelectContext:
        info = SelectInfo(columns=new_columns(*cols))
        context = SelectContext(info=info, db=self)
        context.db = self
        return context

    def save(self, model, *filters: ColumnFilter):
        info = InsertInfo(table=None)
        context = InsertContext(info=info, db=self)
        context.db = self
        return context.save(model, *filters)

    def modify(self, model, *filters: ColumnFilter):
        info = UpdateInfo(table=None)
        context = UpdateContext(info=info, db=self)
        context.db = self
        return context.modify(model, *filters)

    def remove(self, model):
        info = DeleteInfo(table=None)
        context = DeleteContext(info=info, db=self)
        context.db = self
        return context.remove(model)

    def Query(self, cols: Columns, *distinct: bool) -> "SelectClause":
        info = SelectInfo(columns=[])
        context = SelectContext(info=info, db=self)
        context.db = self
        return context.select(cols, *distinct)

    def _new_conn(self) -> IConnection:
        if self.opts.provider == "mysql":
            conn = MysqlConnection(host=self.opts.host, port=self.opts.port
                                       , user=self.opts.user, password=self.opts.password, database=self.opts.database, charset=self.opts.charset)
            self.conn = conn
        elif self.opts.provider == "mssql":
            pass
        elif self.opts.provider == "sqlite":
            pass
        return conn

    @contextlib.contextmanager
    def connection(self):

        conn = self._new_conn()
        try:
            yield conn
        except BeeError as e:
            yield None
        finally:
            conn.close()
            del conn

    @contextlib.contextmanager
    def transaction(self):

        conn = self._new_conn()
        conn.begin()
        try:
            yield
        except BaseException as e:
            conn.rollback()
            conn.close()
            raise e
        finally:
            conn.commit()
            conn.close()
            del conn




from gevent import monkey;monkey.patch_all()
import gevent

import etcd3
from etcd3 import Etcd3Client

from bee.net.rpc.registry.registry import Registry, Server, RegistryError


class Etcd3Registry(Registry):

    def __init__(self, server: Server):
        self.server = server
        self.etcd = self.registry_factory()

    def registry_factory(self) -> Etcd3Client:
        if self.server == None:
            raise RegistryError("load etcd server error")
        address = self.server.address
        if address:
            return etcd3.client(host=address.split(",")[0].split(":")[0], port=int(address.split(",")[0].split(":")[1]))
        else:
            raise RegistryError("load etcd server error")

    def _svc_key(self, service):
        return '/bee/rpc/{}'.format(service)

    def _node_key(self, service, nid):
        return '/bee/rpc/{}/providers/{}'.format(service, nid)

    def register(self, service: str, nid: str, address: str, ttl: int=None):
        n_key = self._node_key(service, nid)
        ttl = self.server.heartbeat_interval
        self.heartbeat(n_key, address, ttl)

    def deregister(self, service: str, nid: str):
        n_key = self._node_key(service, nid)
        self.etcd.delete(key=n_key)

    # def discovery(self, service) -> List[Node]:
    def discovery(self, service) -> dict:
        s_key = self._svc_key(service)
        res = self.etcd.get_prefix(key_prefix=s_key)
        return {bytes(child[1].key).decode().replace(s_key + "/providers/", "") : bytes(child[0]).decode() for child in res}

        # nodes = []
        # for k, v in {bytes(child[1].key).decode().replace(s_key + "/providers/", ""): bytes(child[0]).decode() for child in res}.items():
        #     node = Node(id=k, address=v)
        #     nodes.append(node)
        # return nodes

    def heartbeat(self, key, value, ttl):
        # print("key=%sm value=%s" % (key, value))
        r = self.etcd.put(key, bytes(value, encoding="utf-8"))

        def heartbeat_loop():
            sleep = int(ttl / 2)
            while 1:
                gevent.sleep(sleep)
                # self.etcd.refresh(key, ttl)
                # TODO: refresh
                self.etcd.put(key, value)

        self.beat_thread = gevent.spawn(heartbeat_loop)


    def watch(self, service, callback):

        def watch_loop():

            s_key = self._svc_key(service)
            events_iterator, cancel = self.etcd.watch_prefix(s_key)
            for event in events_iterator:
                callback(event)
        #todo: to be continued
        # self.watch_thread = gevent.spawn(watch_loop)

    def _proc_action(self, action):
        return 'delete' if action == 'expire' else action

    def close(self):
        if hasattr(self, "beat_thread"):
            self.beat_thread.kill()
        if hasattr(self, "watch_thread"):
            self.watch_thread.kill()

def str_to_host(s):
    h, p = s.split(":")
    return (str(h), int(p))


from bee.errors.error import BeeError

class RegistryError(BeeError):
    pass

class Server():

    def __init__(self, protocol: str = None, address: str = None, heartbeat_interval: int = None):
        self.protocol = protocol
        self.address = address
        self.heartbeat_interval = heartbeat_interval


class Registry():

    def register(self, service: str, nid: str, address: str, ttl: int):
        pass

    def deregister(self, service: str, nid: str):
        pass

    def discovery(self, service) -> dict:
        pass

    def close(self):
        pass




from bee.data.map import Map
from bee.db.psd.db import DBOptions, Database
from bee.db.psd.provider.provider import Provider
from bee import config


class Factory():
    name = "psd.Factory"
    """
    Connection factory
    """

    def __init__(self, ):
        self.options = {}
        self.dbMap = Map()

    def open(self, name):
        db = self.dbMap.get(name)
        if not db:
            self.init_db(name)
        db = self.dbMap.get(name)
        return db

    def init_db(self, name):
        db = self.dbMap.get(name)
        if db != None:
            return db

        # 加载配置文件，后面需要提炼出来
        self._load_conf_file()

    def _load_conf_file(self):
        be_data_sql_conf = config.get("bee.data.sql")
        for k, v in be_data_sql_conf.items():
            # options = DBOptions(name=k, provider=v["provider"]
            #                     , max_open_conns=v["max_open_conns"], max_idle_conns=v["max_idle_conns"]
            #                     , trace=v["trace"]
            #                     , options=Map.from_dict(v["options"]))
            options = DBOptions()
            options.name = k
            options.cover(Map.from_dict(v))
            self._add_db(k, options)


    def _add_db(self, name, options=DBOptions()):
        print("opts=(name=%s, provider=%s, max_open_conns=%s, max_idle_conns=%s, trace=%s)" % (options.name, options.provider, options.max_open_conns, options.max_idle_conns, options.trace))
        if options != None:
            self.options[name] = options
            self.dbMap[name] = Factory.build_database(options)

    @staticmethod
    def build_database(options=DBOptions()):
        if options != None:
            database = Database(name=options.name)
            database.opts = options
            database.p = Provider()
            return database

factory = Factory()
class Psd():

    @staticmethod
    def open(db_name) -> Database:
        print(factory)
        db = factory.open(db_name)
        return db




global providers


def register_provider(name, provider_builder):
    global providers
    providers[name] = provider_builder




from gevent import monkey;monkey.patch_all()
import gevent

from consul import Consul
from bee.util.gevent import patch_greenlet
from bee.net.rpc.registry.registry import Registry, Server, RegistryError


class ConsulRegistry(Registry):


    def __init__(self, server: Server):
        self.server = server
        self.client = self.registry_factory()

    def registry_factory(self) -> Consul:
        if self.server == None:
            raise RegistryError("load etcd server error")
        address = self.server.address
        if address:
            return Consul(host=address.split(",")[0].split(":")[0], port=int(address.split(",")[0].split(":")[1]))
        else:
            raise RegistryError("load etcd server error")

    def _svc_key(self, service):
        return '/bee/rpc/{}'.format(service)

    def _node_key(self, service, nid):
        return '/bee/rpc/{}/providers/{}'.format(service, nid)

    def register(self, service: str, nid: str, address: str, ttl: int=None):
        self.client.agent.service.register(name=service, service_id=nid
                                           , address=address.split(",")[0].split(":")[0], port=int(address.split(",")[0].split(":")[1])
                                           )

    @patch_greenlet
    def deregister(self, service: str, nid: str):
        self.client.agent.service.deregister(nid)

    # def discovery(self, service) -> List[Node]:
    def discovery(self, service) -> dict:

        # dns mode
        # from dns import resolver
        # from dns.exception import DNSException
        # consul_resolver = resolver.Resolver()
        # consul_resolver.port = 8600
        # consul_resolver.nameservers = ["127.0.0.1"]
        # dnsanswer_srv = consul_resolver.Query(f"{service}.service.consul", "SRV")
        # print(dnsanswer_srv.response)

        # http mode
        # requests.get(f"http://localhost:8500/v1/catalog/service/{service}")
        res = self.client.catalog.service(service=service)
        services = res[1]
        return {child["ServiceID"] : child["ServiceAddress"] + ":" + str(child["ServicePort"]) for child in services}

    def heartbeat(self, key, value, ttl):
        #todo: to be continued
        pass

    def watch(self, service, callback):
        def watch_loop():
            #todo: to be continued
            print(self.client.health.service(service))
            pass
        self.watch_thread = gevent.spawn(watch_loop)

    def _proc_action(self, action):
        return 'delete' if action == 'expire' else action

    @patch_greenlet
    def close(self):
        if hasattr(self, "beat_thread"):
            self.beat_thread.kill()
        if hasattr(self, "watch_thread"):
            self.watch_thread.kill()


from gevent.lock import BoundedSemaphore
from pymongo import MongoClient
from pymongo.collection import Collection
from pymongo.database import Database
from pymongo.read_preferences import PrimaryPreferred, Secondary, SecondaryPreferred, Nearest, Primary

from bee.data.map import Map
from bee import config

READ_PREFERENCE_PRIMARY = "Primary"
READ_PREFERENCE_PRIMARY_PREFERRED = "PrimaryPreferred"
READ_PREFERENCE_SECONDARY = "Secondary"
READ_PREFERENCE_SECONDARY_PREFERRED = "SecondaryPreferred"
READ_PREFERENCE_NEAREST = "Nearest"


class Options(Map):

    def __init__(self, uri: str = None, max_pool_size: int = 100, min_pool_size: int = 20
                 , socket_time_out: int = 5000, connect_time_out: int = 5000
                 , read_preference: str = READ_PREFERENCE_PRIMARY):
        self.uri = uri
        self.max_pool_size = max_pool_size
        self.min_pool_size = min_pool_size
        self.socket_time_out = socket_time_out
        self.connect_time_out = connect_time_out
        self.read_preference = read_preference


class Client():

    def __init__(self, db: str, c: MongoClient, opts=Options):
        self._db = db
        self._client = c
        self._opts = opts


    def db(self) -> Database:
        return self._client[self._db]

    def coll(self, cool_name: str) -> Collection:
        return self._client[self._db][cool_name]

    def client(self) -> MongoClient:
        return self._client

class Factory():

    def __init__(self):
        self.sem = BoundedSemaphore(1)
        self.clients = Map()

    def open(self, name: str) -> Client:
        self.sem.acquire()
        client = self.clients.get(name)
        if client == None:
            client = self.create(name)
        self.sem.release()
        return client

    def create(self, name: str) -> Client:
        client = self.clients.get(name)
        if client != None:
            return client
        else:
            opts = self.load_options(name)
            if opts == None:
                return
            mode = Primary()
            if opts.read_preference == READ_PREFERENCE_PRIMARY_PREFERRED:
                mode = PrimaryPreferred()
            elif opts.read_preference == READ_PREFERENCE_SECONDARY:
                mode = Secondary()
            elif opts.read_preference == READ_PREFERENCE_SECONDARY_PREFERRED:
                mode = SecondaryPreferred()
            elif opts.read_preference == READ_PREFERENCE_NEAREST:
                mode = Nearest()
            kwargs = {
                "read_preference" : mode,
                "maxPoolSize" : opts.max_pool_size,
                "minPoolSize" : opts.min_pool_size,
                "socketTimeoutMS" : opts.socket_time_out,
                "connectTimeoutMS" : opts.connect_time_out
            }
            _client = MongoClient(host=opts.uri, **kwargs)
            client = Client(db=name, c=_client, opts=opts)
            self.clients.set(name, client)
            return client

    def load_options(self, name) -> Options:
        key = "bee.data.mongo." + name
        if not config.exist(key):
            return None

        opts = Options()
        bee_data_mongo_conf = config.get(key)
        opts.cover(bee_data_mongo_conf)
        return opts



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
warp redis-py 3.0.0(see https://pypi.org/project/redis/)    
        & redis-py-cluster 2.0.0 (see https://pypi.org/project/redis-py-cluster/)
"""

from ._redis import Factory as RedisFactory, Client


from typing import List

from bee.db.psd.clause import InsertClause, InsertColumnsClause, InsertValuesClause, InsertResultClause
from bee.db.psd.column import ColumnFilter
from bee.db.psd.orm import Model


class InsertInfo():
    name = "InsertInfo"

    def __init__(self, table, filter: ColumnFilter=None):
        self.table = table
        self.columns = ()
        self.values = []
        self.filter = filter


class InsertContext(InsertClause, InsertColumnsClause, InsertValuesClause, InsertResultClause):
    name = "InsertContext"

    def __init__(self, info=None, db=None):
        self.info = info
        self.db = db

    def reset(self):
        self.info.columns = None
        self.info.values = None
        self.info.filter = None
        super().reset();
        # super(InsertContext, self).result(); python2.7

    def insert(self, table) -> InsertClause:
        self.info.table = table
        return self

    def save(self, i: Model, *filters: ColumnFilter):
        """
        orm
        :return:
        """
        if len(filters) > 0:
            self.info.filter = filters[0]
        if i != None:
            self.info.table = i.getValue("__table__")

        if self.info.filter == None:
            """
            没有字段filter
            """
            self.info.columns = list(i.keys())
            self.info.values = [list(i.values())]
            self.submit()
        else:
            #TODO:
            pass

    def save_list(self, i: List[Model], *filters: ColumnFilter):
        pass

    def columns(self, *columns: str) -> InsertColumnsClause:
        self.info.columns = columns
        return self

    def values(self, *values) -> InsertValuesClause:
        # print(id(self.info))
        # print(id(self.info.values))
        if self.info.values != None:
            # self.info.values += values
            self.info.values.append(values)
        else:
            self.info.values = list(values)
        return self

    def submit(self):
        return self.result()

    def result(self):
        builder = self.db.p.build_insert(info=self.info)

        return self.db.conn.exec(builder.value(), builder.args)



from bee.db.psd.clause import UpdateClause, SetClause, ResultClause
from bee.db.psd.column import ColumnFilter
from bee.db.psd.criteria import CriteriaSet, SimpleCriteriaSet
from bee.db.psd.orm import Model


class UpdateInfo():
    name = "UpdateInfo"

    def __init__(self, table, where: CriteriaSet = None, filter=None):
        self.table = table
        self.columns = []
        self.values = []
        self.where = where
        self.filter = filter


class UpdateContext(SetClause, UpdateClause, ResultClause):
    name = "UpdateContext"

    def __init__(self, info: UpdateInfo = None, db=None):
        self.info = info
        self.db = db

    def reset(self):
        self.info.columns = None
        self.info.values = None
        self.info.filter = None
        super().reset();

    def update(self, table) -> UpdateClause:
        self.info.table = table
        return self

    def modify(self, i: Model, *filters: ColumnFilter):
        """
        orm
        :return:
        """
        if len(filters) > 0:
            self.info.filter = filters[0]
        if i != None:
            self.info.table = i.getValue("__table__")

        if self.info.filter == None:
            """
            没有字段filter
            """
            self.info.columns = list(i.keys())
            self.info.values = list(i.values())
        else:
            # TODO:
            pass

        # Where
        pk = i.getValue("__primary_key__")
        where = SimpleCriteriaSet()
        where.equal(pk, i.getValue(pk))
        self.info.where = where
        return self.result()



    def set(self, col: str, val) -> SetClause:
        """
        set statement
        :param col:
        :param val:
        :return:
        """
        self.info.columns.append(col)
        self.info.values.append(val)
        return self

    def inc(self, col: str, val: object) -> SetClause:
        pass

    def dec(self, col: str, val: object) -> SetClause:
        pass

    def expr(self, col: str, val: object) -> SetClause:
        pass

    def where(self, where: CriteriaSet) -> ResultClause:
        self.info.where = where
        return self

    def result(self):
        builder = self.db.p.build_update(info=self.info)
        return self.db.conn.exec(builder.value(), builder.args)



from sqlite3 import Connection

from bee.db.psd import IConnection


class SqliteConnection(IConnection):
    def __init__(self, conn=Connection()):
        self.conn = conn

    def exec(self, query, *args):
        self.conn

    def find_one(self, query, *args):
        pass

    def find_list(self, query, *args):
        pass

    def close(self):
        self.conn.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from flask import Flask, Blueprint
from bee import config
from bee.data.guid import Guid
from bee.data.map import Map
from bee.bee import banner

__author__ = 'wenchao.hao'

"""
net.web_flask package. use flask lib as bee's web framework
"""
app = Flask(__name__.split(".")[0])
bapp = Blueprint('bee-' + Guid().string(), __name__)
print(bapp.name)


@bapp.route('/', methods=['GET', 'POST'])
def home():
    s = '<h1>Welcome To Bee Web Home</h1>'
    s += "\n"
    s += "Use Natural Flask Framework, enjoy it!"
    return s


class ServerOptions(Map):

    def __init__(self, port: int = 8080, context_path: str = "bee", banner: bool = True, debug: bool = True):
        self.port = port
        self.path = context_path
        self.banner = banner
        self.debug = debug


def start():
    port = config.get("server.port")
    path = config.get("server.context-path")
    opts = ServerOptions(port=port, context_path=path)
    if opts.debug:
        print(banner)

    app.register_blueprint(bapp, url_prefix=path)
    app.run(debug=opts.debug, use_reloader=False, host='0.0.0.0', port=port)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
db.psd package.
"""

from ._psd import Psd
from .provider.provider import Provider
from .db import DBOptions, Database
from .orm import Model, IntegerField, BooleanField, TextField, StringField, DateTimeField



from bee.data import const
from bee.db.psd.builder import Builder
from bee.db.psd.column import SimpleColumn, ExprColumn, Column
from bee.db.psd.criteria import CriteriaSet, SimpleCriteriaSet, OneColumnCriteria, TwoColumnCriteria, ExprCriteria, \
    NotCriteriaSet, JoinCriteriaSet, CriteriaEnum
from bee.db.psd.delete import DeleteInfo
from bee.db.psd.insert import InsertInfo
from bee.db.psd.select import SelectInfo
from bee.db.psd.table import AliasTable
from bee.db.psd.update import UpdateInfo
from bee.errors.error import BeeError

const.psd_provider_comma = ','
const.psd_provider_dot = '.'

insertValueClauses = (
    "(?)",
    "(?,?)",
    "(?,?,?)",
    "(?,?,?,?)",
    "(?,?,?,?,?)",
    "(?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?)",  # 10
    "(?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",
    "(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)",  # 20
)

class BuildError(BeeError):
    pass

class IProvider():
    name = "psd.provider.IProvider"

    def build_insert(self, info):
        pass

    def build_delete(self, info):
        pass

    def build_update(self, info):
        pass

    def build_select(self, info):
        pass

    def build_call(self, info):
        pass


class Provider(IProvider):

    def quote(self, builder, string):
        # builder.Query += string
        builder.write(string)
        pass

    def limit(self, builder, skip, take):
        pass

    def call(self, builder, sp, args=[]):
        pass

    def build_insert(self, info: InsertInfo = None) -> Builder:
        builder = Builder()

        cols = info.columns.__len__()
        builder.write("INSERT INTO ")
        self.quote(builder, info.table)
        builder.write("(" + info.columns[0])
        for i in range(cols):
            if i > 0:
                builder.write(const.psd_provider_comma)
                self.quote(builder, info.columns[i])
        builder.write(") VALUES ")
        rows = info.values.__len__()
        for i in range(rows):
            if i > 0:
                builder.write(const.psd_provider_comma)
            if i > 20:
                builder.write("( ?")
                for j in range(cols):
                    builder.write(", ?")
                builder.write(")")
            else:
                builder.write(insertValueClauses[cols - 1])
        builder.args = []
        for v in info.values:
            builder.args += v
        # builder.args += v
        return builder

    def build_update(self, info: UpdateInfo = None) -> Builder:

        builder = Builder()
        builder.write("UPDATE ")
        self.quote(builder, info.table)
        builder.write(" SET ")

        cols = info.columns.__len__()
        for i in range(cols):
            if i > 0:
                builder.write(const.psd_provider_comma)
            self.quote(builder, info.columns[i])
            builder.write("=")
            builder.write("?")
        builder.args = info.values
        if hasattr(info, "Where") and info.where != None:
            builder.write(" WHERE ")
            self.build_criteria_set(builder, info.where)
        return builder

    def build_delete(self, info: DeleteInfo = None):

        builder = Builder()
        builder.write("DELETE FROM ")
        self.quote(builder, info.table)

        if not hasattr(info, "Where") or info.where == None:
            raise BuildError("delete action must have Where clause")

        builder.write(" WHERE ")
        self.build_criteria_set(builder, info.where)

        return builder

    def build_select(self, info: SelectInfo = None):
        builder = Builder()
        builder.write("SELECT ")
        if info.distinct != None and info.distinct == True:
            builder.write("DISTINCT ")

        for i, col in enumerate(info.columns):
            if i > 0:
                builder.write(",")
            if col.table() == None:
                if isinstance(col, SimpleColumn):
                    builder.write(col)
                elif isinstance(col, ExprColumn):
                    builder.write(col.expr())
                else:
                    builder.write(col.name())
            else:
                self.quote(builder, col.table().prefix())
                builder.write(const.psd_provider_dot)
                self.quote(builder, col.name())

            if col.alias() !=None and col.alias() != "":
                builder.write_strs(" AS ", col.alias())

        builder.write(" FROM ")
        if isinstance(info.table, AliasTable):
            self.quote(builder, info.table.name())
        else:
            self.quote(builder, info.table)

        if info.table.alias() != None and info.table.alias() != "":
            builder.write_strs(" AS ", info.table.alias())

        self.build_join(builder, info)

        if info.where != None and not info.where.empty():
            builder.write(" WHERE ")
            self.build_criteria_set(builder, info.where)

        self.build_group_by(builder, info)
        return builder

    def build_criteria_set(self, builder, cs: CriteriaSet):
        if isinstance(cs, SimpleCriteriaSet):
            for index, value in enumerate(cs.items):
                if index > 0:
                    builder.write(" AND ")
                if isinstance(value, OneColumnCriteria):
                    self.build_one_column_criteria(builder, value)
                elif isinstance(value, TwoColumnCriteria):
                    self.build_two_column_criteria(builder, value)
                elif isinstance(value, ExprCriteria):
                    builder.write(str(value))

        elif isinstance(cs, NotCriteriaSet):
            builder.write(" NOT")
            self.build_criteria_set(builder, cs.inner)
            builder.write(")")
        elif isinstance(cs, JoinCriteriaSet):
            if (cs.left().empty()):
                self.build_criteria_set(builder, cs.rght())
            elif cs.rght().empty():
                self.build_criteria_set(builder, cs.left())
            else:
                builder.write("(")
                self.build_criteria_set(builder, cs.left())
                builder.write_strs(") ", cs.joiner(), " (")
                self.build_criteria_set(builder, cs.rght())
                builder.write(")")

    def build_one_column_criteria(self, builder, c: OneColumnCriteria):
        if c.table() != None:
            self.quote(builder, c.table().prefix())
            builder.write(const.psd_provider_dot)

        if c.type == CriteriaEnum.NE:
            if c.value == None:
                self.quote(builder, c.col)
                builder.write(" IS NOT NULL")
                return
            self.quote(builder, c.col)
            builder.write("<>?")
        elif c.type == CriteriaEnum.LT:
            self.quote(builder, c.col)
            builder.write("<?")
        elif c.type == CriteriaEnum.LTE:
            self.quote(builder, c.col)
            builder.write("<=?")
        elif c.type == CriteriaEnum.GT:
            self.quote(builder, c.col)
            builder.write(">?")
        elif c.type == CriteriaEnum.GTE:
            self.quote(builder, c.col)
            builder.write(">=?")
        elif c.type == CriteriaEnum.IN:
            self.quote(builder, c.col)
            builder.write(" IN(")
            self.build_in_values(builder, c.value)
            builder.write(")")
        elif c.type == CriteriaEnum.NIN:
            self.quote(builder, c.col)
            builder.write(" NOT IN(")
            self.build_in_values(builder, c.value)
            builder.write(")")
        elif c.type == CriteriaEnum.LK:
            self.quote(builder, c.col)
            builder.write_strs(" LIKE '", c.value, "'")
        else:
            if c.value == None:
                self.quote(builder, c.col)
                builder.write(" IS NOT NULL")
            self.quote(builder, c.col)
            builder.write("=?")
        if (isinstance(c.value, list)):
            builder.args += c.value
        else:
            builder.args.append(c.value)

    def build_in_values(self, builder, val):
        if isinstance(val, list):
            for index, value in enumerate(val):
                if index > 0:
                    builder.write(",")
                # if (isinstance(value, str)):
                #     builder.write_strs("'", value, "'")
                # else:
                #     builder.write(str(value))
                if (isinstance(value, str)):
                    builder.write_strs("'?'")
                else:
                    builder.write("?")


    def build_two_column_criteria(self, builder, c: TwoColumnCriteria):
        if not isinstance(c.left, str) and c.left.table() != None:
            self.quote(builder, c.left.table().prefix())
            builder.write(const.psd_provider_dot)

        if not isinstance(c.left, str):
            self.quote(builder, c.left.name())
        else:
            self.quote(builder, c.left)


        if c.type == CriteriaEnum.EQ:
            builder.write("=")
        elif c.type == CriteriaEnum.NE:
            builder.write("<>")
        elif c.type == CriteriaEnum.LT:
            builder.write("<")
        elif c.type == CriteriaEnum.LTE:
            builder.write("<=")
        elif c.type == CriteriaEnum.GT:
            builder.write(">")
        elif c.type == CriteriaEnum.GTE:
            builder.write(">=")
        else:
            # 默认使用等于
            builder.write("=")

        if hasattr(c.right, "table") and c.right.table() != None:
            self.quote(builder, c.right.table().prefix())
            builder.write(const.psd_provider_dot)
        if isinstance(c.right, str):
            self.quote(builder, c.right)
        else:
            self.quote(builder, c.right.name())

    def build_join(self, builder, info: SelectInfo):
        if info.joins != None and len(info.joins) > 0:
            for i, v in enumerate(info.joins):
                builder.write_strs(" ", v.type, " ")

                if isinstance(v.table, AliasTable):
                    self.quote(builder, v.table.name())
                else:
                    self.quote(builder, v.table)

                if not isinstance(v.table, str):
                    if v.table.alias() != None and v.table.alias() != "":
                        builder.write_strs(" AS ", v.table.alias())
                builder.write(" ON ")
                self.build_criteria_set(builder, v.on)

    def build_group_by(self, builder, info: SelectInfo):
        if info.groups != None and len(info.groups) > 0:
            builder.write(" GROUP BY ")
            for i, col in enumerate(info.groups):
                if i > 0:
                    builder.write(const.psd_provider_comma)
                    self.build_column(builder, col)
            if info.having != None:
                builder.write(" HAVING ")
                self.build_criteria_set(builder, info.having)

    def build_column(self, builder, col: Column):
        if col.table() == None and col.table() == "":
            self.quote(builder, col.name())
        else:
            self.quote(builder, col.table().prefix())
            builder.write(const.psd_provider_dot)
            self.quote(builder, col.name())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
config package. for read yaml、yml、json file.
    像springboot一样读取yaml文件装载到对象options中(but python的dict天生支持json, haha)
"""

from ._config import *


#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'wenchao.hao'
__version__ = '0.1.0'

"""
warp pymongo 3.9.0(see https://pypi.org/project/pymongo/)    
"""

from ._mongo import Factory as MongoFactory, Client


import etcd3
#往etcd中存数据
from etcd3.events import DeleteEvent

client = etcd3.client(host='localhost')   #连接etcd
r  = client.put('aaa', 'qweqwe')              #往etcd中存键值
b = client.get('aaa')                        #查看etcd中的键值

print(r)
print(b)
events_iterator, cancel = client.watch('aaa')
client.put('aaa', '111')
client.delete('aaa')
for event in events_iterator:
    print(event)
    t = type(event)
    if t == DeleteEvent:
        print("删除")

#监听etcd中aaa键 是否发生改变，
for x in client.get_all(keys_only=True):
    print(x[1].key)
#
# # res = client.delete_prefix(prefix="")
# # print(res)
#
# for y in (client.prefix(key_prefix="/bee/rpc/test/providers")):
#     print(y[1])
#
#
# m = {bytes(child[1].key).decode(): bytes(child[0]).decode() for child in client.prefix(key_prefix="/bee/rpc/test/providers")}
# # m = [bytes(child[1].key).decode() for child in client.prefix(key_prefix="/bee/rpc/test/providers")]
# print(m)
# print([x for x in {bytes(child[1].key).decode(): bytes(child[0]).decode() for child in client.prefix(key_prefix="/bee/rpc/test/providers")}.items()])

# client.delete_prefix(prefix="/bee/rpc/test/providers")


from enum import Enum
from typing import List

from bee.db.psd.clause import SelectClause, FromClause, WhereClause, JoinClause, GroupByClause, OrderByClause, HavingClause, \
    SelectResultClause
from bee.db.psd.column import new_expr_column, Columns
from bee.db.psd.criteria import CriteriaSet
from bee.db.psd.table import Table, to_table
from bee.db.psd.value import Value


class OrderEnum(Enum):
    ASC = 0,
    DESC = 1


class Join():
    name = "Join"

    def __init__(self, type: str, t: Table, on: CriteriaSet):
        self.type = type
        self.table = t
        self.on = on


class Order():
    name = "Order"

    def _init__(self, cols: Columns, t: OrderEnum):
        self.columns = cols
        self.type = t


class SelectInfo():
    name = "SelectInfo"

    def __init__(self, table: Table = None, distinct: bool = False, columns: Columns = None, where: CriteriaSet = None,
                 joins: List[Join] = None
                 , groups: object = None, having: CriteriaSet = None, orders: List[Order] = None, skip: int = 0,
                 take: int = 0,
                 count: bool = False):
        self.table = table
        self.distinct = distinct
        self.columns = columns
        self.where = where
        self.joins = []
        self.groups = groups
        self.having = having
        self.orders = orders
        self.skip = skip
        self.take = take,
        self.count = count


class SelectContext(SelectClause, FromClause, WhereClause):
    name = "SelectContext"

    def __init__(self, info: SelectInfo = None, db=None):
        self.info = info
        self.db = db

    def reset(self):
        self.info.distinct = False
        self.info.columns = None
        self.info.where = None
        self.info.joins = None
        self.info.groups = None
        self.info.having = None
        self.info.orders = None
        self.info.skip = 0
        self.info.take = 0
        self.info.count = False
        super().reset()

    def select(self, cols: Columns, *distinct: bool) -> SelectClause:
        self.info.columns = cols
        if distinct != None and len(distinct) > 0:
            self.info.distinct = distinct[0]
        return self

    def count(self, table: Table) -> FromClause:
        self.info.columns = [new_expr_column("COUNT(0)", "count")]
        self.info.table = to_table(table)
        return self

    def From(self, table: Table) -> FromClause:
        self.info.table = to_table(table)
        return self

    def Where(self, w: CriteriaSet) -> WhereClause:
        self.info.where = w
        return self

    def Join(self, t: Table, on: CriteriaSet) -> JoinClause:
        return self._join(t=t, on=on, jt="JOIN")

    def left_join(self, t: Table, on: CriteriaSet) -> JoinClause:
        return self._join(t, on, "LEFT JOIN")

    def right_join(self, t: Table, on: CriteriaSet) -> JoinClause:
        return self._join(t, on, "RIGHT JOIN")

    def full_join(self, t: Table, on: CriteriaSet) -> JoinClause:
        return self._join(t, on, "FULL JOIN")

    def _join(self, t: Table, on: CriteriaSet, jt: str) -> JoinClause:
        join = Join(type=jt, t=t, on=on)
        self.info.joins.append(join)
        return self

    def group_by(self, cols: Columns) -> GroupByClause:
        self.info.groups = cols
        return self

    def having(self, f: CriteriaSet) -> HavingClause:
        self.info.having = f
        return self

    def order_by(self, orders: Order) -> OrderByClause:
        self.info.orders = orders
        return self

    def limit(self, skip: int, take: int) -> SelectResultClause:
        self.info.skip = skip
        self.info.take = take
        return self

    def page(self, index: int, size: int) -> SelectResultClause:
        self.info.skip = (index - 1) * size
        self.info.take = size
        return self

    def value(self) -> Value:
        data = self.row().get("data")
        return data

    def int(self) -> int:
        data = self.row().get("data")
        if data != None and len(data) == 1:
            return data[0]
        else:
            return None

    def one(self, _type: type):
        row = self.row()
        if row == None:
            return None
        cols = row.get("cols")
        data = row.get("data")
        if data == None:
            return None
        ins = _type()
        for i in range(len(cols)):
            col = cols[i]
            # if hasattr(ins, col[0]):
            #     setattr(ins, col[0], data[i])
            setattr(ins, col[0], data[i])
        return ins

    def list(self, _type: type=None):

        rows = self.rows()
        if rows == None:
            return None
        cols = rows.get("cols")
        data_list = rows.get("data")

        if _type == None:
            return data_list

        ins_list = []
        if data_list == None or len(data_list) == 0:
            return []
        for data in data_list:
            ins = _type()
            for i in range(len(cols)):
                col = cols[i]
                # if hasattr(ins, col[0]):
                #     setattr(ins, col[0], data[i])
                setattr(ins, col[0], data[i])
            ins_list.append(ins)
        return ins_list

    def load(self):
        """
            orm
        :return:
        """
        pass

    def row(self):
        builder = self.db.p.build_select(info=self.info)
        return self.db.find_one(builder.value(), builder.args)

    def rows(self):
        builder = self.db.p.build_select(info=self.info)
        print(builder.value())
        return self.db.find_list(builder.value(), builder.args)



from bee.db.psd.clause import DeleteClause, ResultClause
from bee.db.psd.criteria import CriteriaSet, SimpleCriteriaSet
from bee.db.psd.orm import Model


class DeleteInfo():
    name = "DeleteInfo"

    def __init__(self, table: str, where: CriteriaSet = None):
        self.table = table
        self.where = where


class DeleteContext(DeleteClause, ResultClause):
    name = "DeleteContext"

    def __init__(self, info: DeleteInfo = None, db=None):
        self.info = info
        self.db = db

    def reset(self):
        self.info.table = None
        self.info.where = None
        super().reset();

    def delete(self, table) -> DeleteClause:
        self.info.table = table
        return self

    def remove(self, i: Model):
        """
        orm
        :return:
        """

        if i != None:
            self.info.table = i.getValue("__table__")

        self.info.columns = list(i.keys())
        self.info.values = list(i.values())

        # Where
        pk = i.getValue("__primary_key__")
        where = SimpleCriteriaSet()
        where.equal(pk, i.getValue(pk))
        self.info.where = where
        return self.result()

    def where(self, where: CriteriaSet) -> ResultClause:
        self.info.where = where
        return self

    def result(self):
        builder = self.db.p.build_delete(info=self.info)
        return self.db.conn.exec(builder.value(), builder.args)



class Result():
    name = "Result"

    def rowsAffected(self) -> int:
        pass


class InsertResult(Result):
    name = "InsertResult"

    def lastInsertId(self) -> int:
        pass


class ExecuteResult(Result):
    name = "ExecuteResult"

    def lastInsertId(self) -> int:
        pass


class Reader():
    pass



from bee.data.guid import Guid
from bee.net.rpc.registry.consul_registry import ConsulRegistry
from bee.net.rpc.registry.registry import Server

server = Server(protocol="consul", address="127.0.0.1:8500", heartbeat_interval=30)
cr = ConsulRegistry(server)

cr.register(service="test", nid=Guid().string(), address="127.0.01:9999", ttl=server.heartbeat_interval)

services = cr.discovery(service="test")
print(services)

cr.deregister('test', 'bmv9kqb24te0m4j24srg')



import random
from typing import List

from bee.net.rpc.node import Node, NodeState

MaxSelectArraySize = 3
defaultWeight = 1


class RandomBalancer(object):

    def __init__(self, nodes: List[Node] = [], weight=None):
        self.nodes = nodes

    def select(self) -> Node:
        _, ep = select_one_random(self.nodes)
        return ep

    def select_list(self) -> List[Node]:
        index, ep = select_one_random(self.nodes)
        if not ep:
            return ep
        return select_list_from_index(self.nodes, index)


class RoundrobinBalancer(object):

    def __init__(self, nodes: list = [], weight=None):
        self.nodes = nodes
        self.index = 0

    def select(self) -> Node:
        _, ep = self.roundrobin_select()
        return ep

    def select_list(self) -> List[Node]:
        index, ep = self.roundrobin_select()
        if not ep:
            return ep
        return select_list_from_index(self.nodes, index)

    def roundrobin_select(self):
        nodes = self.nodes
        if not nodes:
            return -1, None

        idx = self.index % len(nodes)
        self.index += 1
        if nodes[idx].status == NodeState.Ready:
            return idx, nodes[idx]

        return select_one_random(self.nodes)


def select_one_random(nodes):
    nodes_len = len(nodes)
    if nodes_len == 0:
        return -1, None

    index = random.randint(0, nodes_len - 1)
    if nodes[index].status == NodeState.Ready:
        return index, nodes[index]

    rd = random.randint(0, nodes_len - 1)
    for i in range(nodes_len):
        rdi = (rd + i) % nodes_len
        if nodes[rdi].status == NodeState.Ready:
            return rdi, nodes[rdi]

    return -1, None


def select_list_from_index(nodes, index):
    if not nodes or index < 0:
        return []

    nodes_len = len(nodes)
    ep_list = []
    for i in range(nodes_len):
        idx = (index + i) % nodes_len
        if nodes[idx].status == NodeState.Ready:
            ep_list.append(nodes[idx])
        if len(ep_list) == MaxSelectArraySize:
            break
    return ep_list



import logging

import pymysql

from bee.data.map import Map
from bee.db.psd.connection.connection import IConnection

logging.basicConfig(level=logging.INFO)

class MysqlConnection(IConnection):
    def __init__(self, host: str= '127.0.0.1', port: int=3306, user: str="root", password: str="123456", database: str="test", charset="utf8"):
        logging.info("psd > connect mysql:host=%s, port=%s, user=%s, database=%s, charset=%s" % (host, port, user, database, charset))
        conn = pymysql.connect(host=host, port=port, user=str(user),
                               password=str(password), database=database, charset=charset, autocommit=True)
        self.conn = conn
        conn.open

    def exec(self, query, args=[]):
        # self.conn.begin()
        cursor = self.conn.cursor()
        cursor.execute(query.replace("?", "%s"), args)
        # self.conn.commit()

    def find_one(self, query, args=[]):
        cursor = self.conn.cursor()
        cursor.execute(query.replace("?", "%s"), args)
        cols = cursor.description
        result = Map()
        result.set("cols", cols)
        result.set("data", cursor.fetchone())
        return result

    def find_list(self, query, args=[]):
        cursor = self.conn.cursor()
        cursor.execute(query.replace("?", "%s"), args)
        cols = cursor.description
        result = Map()
        result.set("cols", cols)
        result.set("data", cursor.fetchall())
        return result

    def close(self):
        self.conn.close()

    def begin(self):
        self.conn.begin()

    def commit(self):
        self.conn.commit()

    def rollback(self):
        self.conn.rollback()



import six

request = Request()
request.id = 1000
request.service = "test"
request.method = "test"
request.labels.append(Label(name="label", value="label"))
request.labels.append(Label(name="label1", value="label1"))
request.args.append(b"hello")
request.args.append(b" world")
request.args.append(b"!")
request.args.append(six.int2byte(1))

print(request.SerializeToString())
print(int(2 << 20).bit_length())
print("------------")
# print(bytes("4".encode("utf-8")).decode())
# print(bytearray(4))
# print(six.int2byte(56))
print(bytearray(int(6666).to_bytes(4, byteorder='little')))
print(int.from_bytes(b'\n\x1a\x00\x00', byteorder='little'))
print("------------")
print(hex(1000))


request1 = Request()
request1.ParseFromString(b'\x08\xe8\x07\x12\x04test\x1a\x04test"\x0e\n\x05label\x12\x05label"\x10\n\x06label1\x12\x06label1*\x05hello*\x06 world*\x01!*\x01\x01')
print(request1)

for i in [3,2,5]:
    print(i)

print(bytes("hello world", encoding="utf-8"))

request.ParseFromString(b'\x08\xe8\x07\x12\x04test\x1a\x04test"\x0e\n\x05label\x12\x05label"\x10\n\x06label1\x12\x06label1*\x05hello*\x06 world*\x01!*\x01\x01')
print(request)


import fnmatch
import os
from typing import List


def exist(file_path : str) -> bool:
    return os.path.exists(file_path)

def not_exist(file_path : str) -> bool:
    return not os.path.exists(file_path)


def is_dir(file_path : str) -> bool:
    return os.path.isdir(file_path)

def is_file(file_path : str) -> bool:
    return os.path.isfile(file_path)

def all_file(top: str, *patterns: str) -> List[str]:
    _files = []
    for root, dirs, files in os.walk(top, topdown=True):
        files.sort()
        for fname in files:
            for pt in patterns:
                if fnmatch.fnmatch(fname, pt):
                    _files.append(os.path.join(root, fname))

    return _files

def file_name(file_path : str) -> str:
    if is_file(file_path):
        return os.path.basename(file_path)
    return None






from io import StringIO


class Builder():
    def __init__(self):
        self.query = StringIO("")
        self.args = []

    def reset(self):
        self.query = None
        if self.args != None:
            self.args = self.args[:0]

    def write(self, string):
        self.query.write(string)

    def write_strs(self, *strs):
        for i in strs:
            self.query.write(i)

    def value(self):
        return self.query.getvalue()

    def fmt_value(self):
        return self.query.getvalue().replace("?", "%s")



import os
import sys
from typing import List

from bee.ext import files as files
from bee.config.source import EnvSource, Source, cover_option, FileSource, ByteDataSource
from bee.data.map import Map


exts: List[str] = [".yml", ".yaml", ".json"]


class Unmarshaler():
    """
    Unmarshaler is custom unmarshal interface for Config.Unmarshal method.
    """

    def Unmarshal(self):
        pass


class Manager():

    def __init__(self):
        self.loaded: bool = False
        self.autoLoad: bool = False

        self.env: EnvSource = EnvSource()

        self.options: Map = None
        self.profiles: List[str] = []
        self.dirs: List[str] = []
        self.name: str = None
        self.srcs: List[Source] = []

        self.defaults: Map = Map()

    def set_name(self, name: str):
        self.name = name

    @staticmethod
    def new(name: str) -> "Manager":
        m = Manager()
        if not name:
            m.set_name(name)
        else:
            m.set_name("app")

        m.set_default_value("banner", True)
        m.set_env_prefix("BEE")
        return m

    def set_default_value(self, name: str, value):
        # if not self.loaded:
        #     self.load(True)
        cover_option(self.defaults, name.lower(), value)

    def set_env_prefix(self, prefix: str):
        self.env.set_prefix(prefix)

    def add_folder(self, *dirs: str):
        length = len(dirs)
        if length > 0:
            for v in dirs:
                self.dirs.append(v)

    def find_file(self, name: str, *exts) -> str:
        """
        find_file searches all config directories and return the first found file.
        :param name:
        :param exts:
        :return:
        """
        for dir in self.dirs:
            for ext in exts:
                path = dir + "/" + name + ext
                if files.exist(path):
                    return path
                else:
                    return ""

    def find_files(self, name: str, *exts) -> List[str]:
        """
        find_files searches all config directories and return all found files.
        :param name:
        :param exts:
        :return:
        """
        fs = []
        for dir in self.dirs:
            for ext in exts:
                path = dir + "/" + name + ext
                if files.exist(path):
                    fs.append(path)
        return fs

    def find_folder(self, name) -> str:
        """
        find_folder earches all config directories and return the first found folder.
        :param name:
        :return:
        """
        for dir in self.dirs:
            path = dir + "/" + name
            if files.exist(path):
                return path
            else:
                return ""

    def find_folders(self, name) -> List[str]:
        dirs = []
        for dir in self.dirs:
            path = dir + "/" + name
            if files.exist(path):
                dirs.append(path)
        return dirs

    def load(self, force: bool):
        if self.loaded and not force:
            return None
        self.options = Map()
        """read env source"""
        self.load_source(self.env)

        """read file source"""
        srcs = self.find_file_sources()
        self.load_source(*srcs)

        """read self.srcs"""
        self.load_source(*self.srcs)

        self.loaded = True

    def load_source(self, *srcs: Source):
        for src in srcs:
            opts = src.load()
            self.options.merge(opts)

    def get(self, key: str):
        if not self.loaded:
            self.load(True)
        opt = self.options.find(key)
        default = self.defaults.find(key)
        if default == None:
            return opt
        elif opt == None:
            return default
        return opt

    def find_file_sources(self) -> List[Source]:
        srcs = []
        # step 1: if len(self.dirs)==0, read default folders
        if len(self.srcs) == 0:
            # TODO: to be continued...
            self.add_folder(os.path.join(sys.path[1], "config"))
            self.add_folder(".")

        # step 2: else read self.dirs
        for dir in self.dirs:
            for ext in exts:
                for profile in self.profiles:
                    path = dir + "/" + self.name + "." + profile + ext
                    if files.exist(path):
                        srcs.append(FileSource(path))

        for dir in self.dirs:
            for ext in exts:
                path = dir + "/" + self.name + ext
                if files.exist(path):
                    srcs.append(FileSource(path))
        return srcs

    def set_profiles(self, *profiles: str):
        for profile in profiles:
            self.profiles.append(profile)

    def add_source(self, *srcs: Source):
        for src in srcs:
            self.srcs.append(src)

    def add_byte_source(self, d: bytes, t: str):
        ds = ByteDataSource(d, t)
        self.add_source(ds)

    def bind_env(self, key: str, env_key: str):
        self.env.set_aliases(key, env_key)





import json
from typing import List

from bee.data.option import Options, Option
from bee.data.map import Map
from bee.errors.error import CodedError
from bee.net.rpc.codec import CodecBuilder, Request as _Request, RequestHead as _RequestHead, ResponseHead as _ResponseHead, \
    Response as _Response, Result as _Result, register_codec, IClientCodec, Stream, IServerCodec


class Request():
    name = "request"
    def __init__(self):
        self.head = _RequestHead
        self.args: List[bytes] = []

    def reset(self):
        self.head.id = 0
        self.head.service = ""
        self.head.method = ""
        if self.head.labels != None:
            self.head.labels = self.head.labels[:0]

        if self.args != None:
            self.args = self.args[:0]

class Result():
    name = "result"
    def __init__(self):
        self.value: bytes = None
        self.error: CodedError = None


class Response():
    name = "response"

    def __init__(self):
        self.head: _ResponseHead = None
        self.result: Result = None

    def reset(self):
        self.head = 0
        self.result.value = None
        self.result.error = None

class ClientCodec(IClientCodec):
    """
    客户端编解码
    """

    def __init__(self, s: Stream, req=Request(), resp: Response = None):
        self.s = s
        self.req: Request = req
        self.resp: Response = resp

    def stream(self) -> Stream:
        return self.s

    def encode(self, req: _Request):

        json_str = json.dumps(req, default=lambda o: o.__dict__, sort_keys=True)
        print(json_str)
        data = json_str.encode(encoding="utf-8")
        self._write(data)

    def _write(self, data: bytes):
        # wtire data's length, occupy 4 bytes

        self.s.write(len(data).to_bytes(4, byteorder='little'))
        # write data
        self.s.write(data)
        self.s.flush()

    def discard_result(self):
        return None

    def decode_head(self, head: _ResponseHead):
        length_bytes = self.s.read(4);
        length = int.from_bytes(length_bytes, byteorder="little")

        data = self.s.read(length)
        # self.resp = json.loads(bytes(data).decode(encoding="utf-8"), cls=Response)
        # self.resp = json.loads(bytes(data).decode(encoding="utf-8"), cls=Response)
        result = json.loads(bytes(data).decode(encoding="utf-8"))
        print(result)
        _head = _ResponseHead(id=result["head"]["id"])
        _result = _Result(value=result["result"]["value"], error=result["result"]["error"])
        self.resp.head = _head
        self.resp.result = _result

        head.id = self.resp.head.id


    def decode_result(self, result: _Result):
        if self.resp.result.value != None:
            result.error = None
            result.value = self.resp.result.value
        else:
            result.error = self.resp.result.error


class ServerCodec(IServerCodec):
    """
    服务端编解码
    """

    def __init__(self, s: Stream, req: Request = None, resp: Response = None):
        self.s = s
        self.req = req
        self.resp = resp

    def stream(self) -> Stream:
        return self.s

    def encode(self, resp: _Response):
        self.resp.head = resp.head

        result = Result()
        if resp.result.error == None:
            if resp.result.value != None:
                result.value = resp.result.value
        else:
            result.error = resp.result.error
        self.resp.result = result
        json_str = json.dumps(self.resp, default=lambda o: o.__dict__, sort_keys=True)
        # print(json_str)
        data = json_str.encode(encoding="utf-8")
        self._write(data)

    def _write(self, data: bytes):
        # wtire data's length, occupy 4 bytes
        self.s.write(len(data).to_bytes(4, byteorder='little'))
        # write data
        self.s.write(data)
        self.s.flush()

    def decode_head(self, head: _RequestHead):
        length_bytes = self.s.read(4);
        length = int.from_bytes(length_bytes, byteorder="little")
        if length ==0:
            return
        data = self.s.read(length)
        # print(data)
        result = json.loads(data)
        self.req.args = result["args"]
        _head = _RequestHead()
        _head.id = result["head"]["id"]
        _head.service = result["head"]["service"]
        _head.method = result["head"]["method"]
        self.req.head = _head

        head.id = self.req.head.id
        head.service = self.req.head.service
        head.method = self.req.head.method
        if self.req.head.labels:
            opts = Options()
            for v in self.req.head.labels:
                opts.append(Option(name=v.name, value=v.value))
            head.labels = opts
        if head.id != None and head.id > 0:
            print("%s, %s.%s(%s), labels(%s)" % (head.id, head.service, head.method, "", head.labels))

    def decode_args(self, args: List[object]):
        if self.req.args != None:
            for i, v in enumerate(self.req.args):
                args.insert(i, v)

    def discard_args(self):
        return None


class Builder(CodecBuilder):

    name="json"

    def new_client_codec(self, s: Stream, opts: Map) -> ClientCodec:
        return ClientCodec(s=s, req=Request(), resp=Response())

    def new_server_codec(self, s: Stream, opts: Map) -> ServerCodec:
        return ServerCodec(s=s, req=Request(), resp=Response())


def init():
    register_codec("json", Builder())


#
# from db.psd.connection.connection import IConnection
#
#
# class MssqlConnection(IConnection):
#     def __init__(self, conn=Connection()):
#         self.conn = conn
#
#     def exec(self, Query, *args):
#         self.conn
#
#     def find_one(self, Query, *args):
#         pass
#
#     def find_list(self, Query, *args):
#         pass



import logging

from bee.errors.error import BeeError


class StandardError(BeeError):
    pass

class Field(object):

    def __init__(self, name, column_type, primary_key, default):
        self.name = name
        self.column_type = column_type
        self.primary_key = primary_key
        self.default = default

    def __str__(self):
        return '<%s, %s:%s>' % (self.__class__.__name__, self.column_type, self.name)


class StringField(Field):

    def __init__(self, name=None, primary_key=False, default=None, ddl='varchar(255)'):
        super().__init__(name, ddl, primary_key, default)


class BooleanField(Field):

    def __init__(self, name=None, default=False):
        super().__init__(name, 'boolean', False, default)


class IntegerField(Field):

    def __init__(self, name=None, primary_key=False, default=0):
        super().__init__(name, 'bigint', primary_key, default)


class FloatField(Field):

    def __init__(self, name=None, primary_key=False, default=0.0):
        super().__init__(name, 'real', primary_key, default)


class TextField(Field):

    def __init__(self, name=None, default=None):
        super().__init__(name, 'text', False, default)

class DateTimeField(Field):

    def __init__(self, name=None, default=None):
        super().__init__(name, 'datetime', False, default)

class ModelMetaclass(type):

    def __new__(cls, name, bases, attrs):
        if name == 'Model':
            return type.__new__(cls, name, bases, attrs)
        tableName = attrs.get('__table__', None) or name
        logging.info('psd > found model: %s (table: %s)' % (name, tableName))
        mappings = dict()
        fields = []
        primaryKey = None
        for k, v in attrs.items():
            if isinstance(v, Field):
                logging.info('psd > found mapping: %s ==> %s' % (k, v))
                mappings[k] = v
                if v.primary_key:
                    # 找到主键:
                    if primaryKey:
                        raise StandardError('Duplicate primary key for field: %s' % k)
                    primaryKey = k
                else:
                    fields.append(k)
        if not primaryKey:
            raise StandardError('Primary key not found.')
        for k in mappings.keys():
            attrs.pop(k)
        attrs['__mappings__'] = mappings  # 保存属性和列的映射关系
        attrs['__table__'] = tableName
        attrs['__primary_key__'] = primaryKey  # 主键属性名
        attrs['__fields__'] = fields  # 除主键外的属性名
        return type.__new__(cls, name, bases, attrs)


class Model(dict, metaclass=ModelMetaclass):

    def __init__(self, **kw):
        super(Model, self).__init__(**kw)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError(r"'Model' object has no attribute '%s'" % key)

    def __setattr__(self, key, value):
        self[key] = value

    def getValue(self, key):
        return getattr(self, key, None)

    def getValueOrDefault(self, key):
        value = getattr(self, key, None)
        if value is None:
            field = self.__mappings__[key]
            if field.default is not None:
                value = field.default() if callable(field.default) else field.default
                logging.debug('using default value for %s: %s' % (key, str(value)))
                setattr(self, key, value)
        return value



import json
import os

import yaml

from bee.data.map import Map
from bee.data.option import Option
from bee.errors.error import BeeError

class SourceTypeError(BeeError):
    pass

class Source():

    def load(self) -> Map:
        pass


# def merge_option(opts: Map, k: str, v):
#     keys = k.split(".")
#     new_keys = []
#     for value in keys:
#         if value != "":
#             new_keys.append(value)
#     keys = new_keys
#     length = len(keys)
#     last = length -1
#     for i in range(length):
#
#         key = keys[i]
#         if opts.contains(key):
#             opt = opts.get(key)
#             t = type(opt)
#             if t == Map:
#                 opts = opt
#             elif t == dict:
#                 opts = Map.from_dict(opt)
#             else:
#                 return
#         else:
#             if i == last:
#                 opts[key] = v
#             else:
#                 opts[key] = Map()


def merge_option(opts: Map, k: str, v):
    keys = k.split(".")
    new_keys = []
    for value in keys:
        if value != "":
            new_keys.append(value)
    keys = new_keys
    length = len(keys)
    last = length - 1
    if length > 1:
        for i in range(length):
            key = keys[i:i+1][0]
            if i < last:
                if opts.get(key) == None:
                    opts = opts.setdefault(key, Map())
                else:
                    opts = opts.get(key)
                    if opts.contains(keys[i+1:i+2][0]):
                        if i == last-1:
                            return
                        else:
                            t = type(opts.get(keys[i+1:i+2][0]))
                            if t!=Map and t!=dict:
                                return
            else:
                opts.setdefault(key, v)

    else:
        if opts.contains(k) == False:
            opts.set(k, v)

def cover_option(opts: Map, k: str, v):
    # print("opts=%s, k=%s, v=%s" % (opts, k, v))
    # keys = k.split(".")
    # length = len(keys)
    # last = length -1
    # for i in range(length):
    #     key = keys[i]
    #     if opts.contains(key):
    #         opt = opts.get(key)
    #         t = type(opt)
    #         if t == Map:
    #             pass
    #         elif t == map:
    #             pass
    #         else:
    #             return
    #     if i == last:
    #         opts[key] = v
    #     else:
    #         opts = opts.setdefault(key, Map())
    keys = k.split(".")
    new_keys = []
    for value in keys:
        if value != "":
            new_keys.append(value)
    keys = new_keys
    length = len(keys)
    last = length - 1
    if length > 1:
        for i in range(length):
            key = keys[i:i + 1][0]
            if i < last:
                if opts.get(key) == None:
                    opts = opts.setdefault(key, Map())
                else:
                    opts = opts.get(key)
                    print(opts)
            else:
                opts.setdefault(key, v)

    else:
        opts.set(k, v)

class FileSource(str):

    def load(self) -> Map :
        path = str(self)

        with open(path, "rb") as f:
            file_name = f.name
            last_dot_index = file_name.rfind(".")
            t = file_name[last_dot_index+1:]
            b = f.read()
            bds = ByteDataSource(d=b, t=t)
            return bds.load()

class EnvSource(Source):

    def __init__(self, prefix: str=None, aliases: Map=None):
        self.prefix = prefix
        self.aliases = aliases

    def load(self) -> Map :
        opts = Map()
        envs = os.environ
        for k, v in envs.items():
            opt = Option(name=k, value=v)
            key = opt.name.lower()
            if self.prefix != None and self.prefix != "":
                key = key.strip(self.prefix).lower()
            # key = opt.name.replace("_", ".").lower()
            # print("key=%s, value=%s" % (key, opt.value))
            merge_option(opts, key, opt.value)
        # print(opts)
        return opts

    def set_prefix(self, prefix: str=None):
        self.prefix = prefix

    def set_aliases(self, alias: str=None, key: str=None):
        if self.aliases == None:
            self.aliases = Map()
        self.aliases.set(alias, key)


def load_source(t: str, d: bytes) -> Map:
    if t == "yaml" or t == "yml":
        return Map.from_dict(d=yaml.load(d, Loader=yaml.FullLoader))
    elif t == "json":
        return Map.from_dict(d=json.loads(d))
    else:
        raise SourceTypeError("unsupported config type")


class ByteDataSource(Source):

    def __init__(self, d: bytes, t: str):
        self.data: bytes = d
        self.type: str = t

    def load(self) -> Map:
        return load_source(self.type, self.data)


from gevent import monkey;monkey.patch_all()
import gevent
from gevent import signal
from gevent.server import StreamServer

from datetime import datetime
from typing import List
import logging
from bee.net.rpc.registry.builder import Builder
from bee.net.rpc.registry.registry import Server as RegistryServer

from bee.net.rpc.channel import Channel
from bee.net.rpc.codec import RequestHead, IServerCodec, Response, ResponseHead, Result, \
    Stream, get_codec
from bee.net.rpc.registry.direct_registry import DirectRegistry
from bee.net.rpc.transport import Address

from bee.data.guid.guid import Guid
from bee.data.map import Map
from bee.errors.error import BeeError
from bee.net.rpc.registry.registry import Registry
from bee.data import const

from bee import config

logging.basicConfig(level=logging.DEBUG)
is_debug = True


class ServerLoadError(BeeError):
    pass


class ReadPeeker(object):
    pass


class MatchInfo():
    """
    codec macher
    """

    def __init__(self, codec: str = "proto", opts: Map = None):
        self.codec = codec
        self.opts = opts

    def match(self, p: ReadPeeker) -> bool:
        """
        偷瞄两眼，看看是什么编码，吼吼吼
        :param p:
        :return:
        """
        pass


class Session():

    def id(self) -> str:
        pass

    def local_addr(self) -> str:
        pass

    def remote_addr(self) -> str:
        pass


class session(Session):

    def __init__(self, ch: Stream, sc: IServerCodec, heartbeat: datetime = datetime.now()):
        self.channel = ch
        self.sc = sc
        self.heartbeat = heartbeat

    def id(self) -> str:
        return self.channel.id()


def new_session(ch: Stream, sc: IServerCodec) -> session:
    return session(ch=ch, sc=sc, heartbeat=datetime.now())


class SessionMap():

    def __init__(self):
        self.channels = Map()

    def add(self, s: session):
        self.channels.set(s.id(), s)

    def get(self, id: str):
        self.channels.get(id)

    def remove(self, id: str):
        self.channels.remove(id)

    def count(self):
        return len(self.channels)


class ServerOptions(Map):

    def __init__(self, name: str = None, desc: str = None, version: str = "1.0", macher: str = "proto",
                 address: Address = None
                 , max_conn_size: int = 1024, max_pool_size: int = 1024, backlog: int = 1000
                 , read_timeout: int = 10, write_timeout: int = 10):
        self.id = Guid().string()
        self.name = name
        self.desc = desc
        self.version = version
        self.macher = macher
        self.address = address
        self.max_conn_size = max_conn_size
        self.max_pool_size = max_pool_size
        self.backlog = backlog
        self.read_timeout = read_timeout
        self.write_timeout = write_timeout


class Server():

    def __init__(self, opts: ServerOptions, registry: Registry = None):
        self.opts = opts

        if registry == None:
            self.registry = DirectRegistry(url=opts.address.url)
        else:
            self.registry = registry

        # 初始化service管理容器
        self.services = Map()

        # 初始化session管理容器
        self.sessions = SessionMap()

        # 初始化编解码器
        self.init_codec_builder()


    @staticmethod
    def new_server(name: str = None, macher: str = "proto", address: Address = Address(url="127.0.0.1:9000"),
                   registry: Registry = None) -> "Server":
        opts = ServerOptions(name=name, macher=macher, address=address,)
        if is_debug:
            logging.debug("bee.rpc > init server's options")
        s = Server(opts=opts, registry=registry)
        if is_debug:
            logging.debug("bee.rpc > new server...")
        return s

    def register_service(self, ins):
        """register rpc service"""
        if not ins:
            return
        name = ins.__class__.__name__
        self.services.set(name, ins)

    def handle(self, sock, address):
        print("address= " + str(address))
        while True:
            if sock.closed == True:
                print("closed...")
                break
            self.handle_request(sock, address)
        sock.close()

    def init_codec_builder(self):
        if self.opts.macher == "http":
            pass
        elif self.opts.macher == "json":
            b = get_codec("json")
            if not b:
                import bee.net.rpc.codecs.json.json as json
                json.init()
        else:  # proto
            b = get_codec("proto")
            if not b:
                import bee.net.rpc.codecs.proto.proto as proto
                proto.init()

    def new_server_codec(self, sock) -> IServerCodec:
        m = self.opts.macher
        if m == None:
            raise ServerLoadError("no macher!")
        else:
            b = get_codec(m)
            if not b:
                raise ServerLoadError("init codec builder error!")
            return b.new_server_codec(s=Channel(id=Guid().string(), sock=sock), opts=Map())

    def add_session(self, sc) -> session:
        s = new_session(sc.stream(), sc)
        self.sessions.add(s)
        return s

    def remove_session(self, sc) -> session:
        self.sessions.remove(sc.stream().id())

    def handle_request(self, sock, address):
        rh = RequestHead()
        args = []
        sc = self.new_server_codec(sock)
        sc.decode_head(rh)
        if rh.id == None or rh.id == 0:
            sock.close()
        else:
            # print("rh.id=" + str(rh.id))
            sc.decode_args(args)
            self.add_session(sc)
            self.invoke_service(sc, rh=rh, args=args)
            self.remove_session(sc)

    def invoke_service(self, sc, rh: RequestHead, args: List):
        id = rh.id
        sname = rh.service
        s = self.services.get(sname)
        mname = rh.method
        if s != None:
            ret = getattr(s, mname)(*(args))
            head = ResponseHead(id=id, assets=None)
            result = Result(value=ret, error=None)
            resp = Response(head=head, result=result)
            sc.encode(resp)
        else:
            print("do nothing")

    def find_codec_builder(self, macher: str) -> IServerCodec:
        cb = const.codecs.get(macher)
        return cb

    def register(self):
        self.nid = Guid().string()
        self.registry.register(self.opts.name, self.nid, self.opts.address.url)

    def handle_signal(self):
        """register signal"""

        def handler(signum, frame):
            logging.info("bee.rpc > kill service={}, nid={}".format(self.opts.name, self.nid))
            self.registry.deregister(self.opts.name, self.nid)

        signal.signal(signal.SIGINT, handler)
        signal.signal(signal.SIGTERM, handler)

    def startup(self):
        """
        start the rpc server
        """
        # if not self.handler:
        #     raise ServerLoadError('Methods not exits.')

        self.handle_signal()

        pair = self.opts.address.url.split(":")
        server = StreamServer(
            (pair[0], int(pair[1])),
            self.handle,
            backlog=self.opts.backlog,
            spawn=self.opts.max_conn_size)

        self.register()

        server.serve_forever()
        self.srv = server

    def shutdown(self):
        """
        shutdown the rpc server
        """
        if self.registry != None:
            self.registry.close()

        if self.srv != None:
            self.srv.close()

    def __call__(environ, start_response):
        # for gunicorn wsgi app
        pass

def auto_server():

    if config.exist("debug"):
        is_debug = config.get("debug")
    bee_rpc_registry_conf = config.get("bee.rpc.registry")
    registry_server = RegistryServer()
    registry_server.__dict__ = bee_rpc_registry_conf
    registry = Builder.build(registry_server)

    bee_rpc_server_conf = config.get("bee.rpc.server")
    if bee_rpc_server_conf != None:
        for k, v in dict(bee_rpc_server_conf).items():
            opts = ServerOptions()
            opts.name = k
            opts.cover(Map.from_dict(v))
            srv = Server(opts=opts, registry=registry)
            srv.startup()

            logging.info("bee.rpc > server start success")




from bee.net.rpc.registry.consul_registry import ConsulRegistry
from bee.net.rpc.registry.etcdv3_registry import Etcd3Registry
from bee.net.rpc.registry.registry import Registry, Server


class Builder():

    @staticmethod
    def build(server: Server) -> Registry:
        if server.protocol == "etcd3":
            return Etcd3Registry(server)
        elif server.protocol == "consul":
            return ConsulRegistry(server)
        elif server.protocol == "zookeeper":
            pass


from bee.db.psd.exector import IExecutor
from bee.errors.error import BeeError


class ConnectionError(BeeError):
    pass

class IConnection(IExecutor):

    def exec(self, query, args=[]):
        raise ConnectionError("no effective db connection，please use 'with db.connection() as conn:' syntax")

    def close(self):
        pass

    def begin(self):
        pass

    def commit(self):
        pass

    def rollback(self):
        pass


class Table():
    def name(self) -> str:
        pass

    def alias(self) -> str:
        pass

    def prefix(self) -> str:
        pass


def to_table(table: object) -> "Table":
    if isinstance(table, str):
        return SimpleTable(table)
    elif isinstance(table, Table):
        return table


def new_table(name: str, *alias) -> "Table":
    if len(alias) == 0:
        return SimpleTable(name)
    else:
        return new_alias_table(name, alias[0])


class SimpleTable(str, Table):

    def __init__(self, t: str):
        self.t = t

    def name(self) -> str:
        return self.t

    def alias(self) -> str:
        return ""

    def prefix(self) -> str:
        return self.t


class AliasTable(Table):

    def __init__(self, name: str, alias: str, prefix: str):
        self._name = name
        self._alias = alias
        self._prefix = prefix

    def name(self) -> str:
        return self._name

    def alias(self) -> str:
        return self._alias

    def prefix(self) -> str:
        return self._prefix


def new_alias_table(name, alias):
    prefix = ""
    if alias == "":
        prefix = name
    else:
        prefix = alias
    return AliasTable(name, alias, prefix=prefix)



from enum import Enum
from typing import List

from gevent.lock import BoundedSemaphore


from bee.data.option import Options
from bee.data.map import Map
from bee.errors.error import BeeError
from bee.net.rpc.balancer import RoundrobinBalancer, RandomBalancer
from bee.net.rpc.codec import get_codec, Result
from bee.net.rpc.node import Node
from bee.net.rpc.registry.builder import Builder
from bee.net.rpc.registry.direct_registry import DirectRegistry
from bee.net.rpc.registry.builder import Registry, Server
from bee.net.rpc.transport import Address

from bee import config

"""default max retry times"""
default_max_retries = 3

class RemoteError(BeeError):
    pass

class FailMode(Enum):
    """
    fail mode
    """
    """FailFast returns error immediately"""
    FailFast = 0
    """FailOver selects another server automatically"""
    FailOver = 1
    """FailTry use current client again"""
    FailTry = 2



class CodecOption(Map):

    def __init__(self, name: str = None, options: Options = None):
        self.name: str = name
        self.options: Options = options


class ClientOptions(Map):

    def __init__(self, name: str = None, version: str = None, fail: str = "failover", address: Address = None
                 , codec: CodecOption = CodecOption(name="proto"), balancer: str = "random"
                 , connect_timeout: int = 5, read_timeout: int = 10, write_timeout: int = 10):
        self.name = name
        self.version = version
        self.fail = fail
        self.address = address
        self.codec = codec
        self.balancer = balancer
        self.connect_timeout = connect_timeout
        self.read_timeout = read_timeout
        self.write_timeout = write_timeout


class Client():

    def __init__(self, opts: ClientOptions, registry: Registry = None):

        # print("opts={name=%s, version=%s, fail=%s, address=%s, codec=%s, balancer=%s}" % (opts.name, opts.version, opts.fail, opts.address, opts.codec.name, opts.balancer))

        self.opts = opts

        if registry == None:
            self.registry = DirectRegistry(url=opts.address.url)
        else:
            self.registry = registry

        # init codec
        self.init_codec_builder()

        self.init_nodes()

        # init cluster's fail mode
        self.init_fail()

        # init balancer
        self.init_balancer(self.nodes)

        self.watch()

    def init_nodes(self):
        n_dict = self.registry.discovery(self.opts.name)
        nodes = []
        for k, v in n_dict.items():
            node = Node(id=k, address=v, cb=get_codec(self.opts.codec.name))
            nodes.append(node)
        self.nodes = nodes


    def init_codec_builder(self):
        if self.opts.codec.name == "http":
            pass
        elif self.opts.codec.name == "json":
            b = get_codec("json")
            if not b:
                import bee.net.rpc.codecs.json.json as json
                json.init()
        else:  # proto
            b = get_codec("proto")
            if not b:
                import bee.net.rpc.codecs.proto.proto as proto
                proto.init()

    def init_fail(self):
        if self.opts.fail == "failover" :
            self.fail_mode = FailMode.FailOver
        elif self.opts.fail == "failfast" :
            self.fail_mode = FailMode.FailFast
        else:
            self.fail_mode = FailMode.FailTry

    def init_balancer(self, nodes):
        if self.opts.balancer == None or self.opts.balancer == "round-robin":
            self._lb = RoundrobinBalancer(self.nodes)
        if self.opts.balancer == "random":
            self._lb = RandomBalancer(self.nodes)

    def watch(self):
        self.registry.watch(self.opts.name, self.notify)

    def notify(self, event):
        print("event=%s" % str(event))

    @staticmethod
    def new_client(name: str = None, address: Address = Address(url="127.0.0.1:9000"),
                   registry: Registry = None) -> "Client":
        opts = ClientOptions(name=name, address=address)
        return Client(opts=opts, registry=registry)

    def call(self, service: str, method: str, args: List[object]) -> Result:

        if self.fail_mode == FailMode.FailFast:
            return self.retry(retry=0, retried=0, change_node=False, call_param=(service, method, args))
        elif self.fail_mode == FailMode.FailOver:
            return self.retry(retry=default_max_retries, retried=0, change_node=True, call_param=(service, method, args))
        else:
            return self.retry(retry=default_max_retries, retried=0, change_node=False, call_param=(service, method, args))



    def retry(self, retry=0, retried=0, change_node=False, call_param:tuple=()):
        """

        :param retry: 重试次数
        :param retried: 已重试次数
        :param change_node: 是否切换节点
        :param call_param: 调用参数(元组) service, method, args
        :return:
        """
        node = self._lb.select()
        try:
            if not node:
                raise RemoteError("no server's ready")
            result = node.call(service=call_param[0], method=call_param[1], args=call_param[2])
            return result
        except Exception as e:
            if retry > 0 and retried < retry:
                return self.retry(retry, retried, change_node, call_param)
            raise RemoteError("no server's ready")

class Cluster(object):
    """
        client-side cluster
        manage all client
    """

    def __init__(self):
        """Cluster 抽象"""
        self.sem = BoundedSemaphore(1)
        self._clients = {}
        self.init_config()

    def get_client(self, service) -> Client:
        if not service in self._clients:
            self.sem.acquire()
            if not service in self._clients:
                self._clients[service] = Client(self.context, service)
            self.sem.release()
        return self._clients[service]

    def init_config(self):
        bee_rpc_registry_conf = config.get("bee.rpc.registry")
        server = Server()
        server.__dict__ = bee_rpc_registry_conf
        self.registry = Builder.build(server)

        bee_rpc_client_conf = config.get("bee.rpc.client")
        if bee_rpc_client_conf != None:
            for k, v in dict(bee_rpc_client_conf).items():
                opts = ClientOptions()
                opts.name = k
                opts.cover(Map.from_dict(v))
                self._clients[k] = Client(opts=opts, registry=self.registry)





from gevent import monkey;monkey.patch_all()
from gevent import socket

from bee.data.guid.guid import Guid
from bee.net.rpc.codec import Stream


class Channel(Stream):

    def __init__(self, id: str = None, sock=None):
        self._id = id
        self._r: socket = sock
        self._w: socket = sock
        #u 用户
        self.u = None

    def id(self) -> str:
        return self._id

    def reader(self) -> socket:
        return self._r

    def writer(self) -> socket:
        return self._w

    def peek(self, n: int) -> bytes:
        """
        瞄一眼
        :param n:
        :return:
        """
        pass


    def read(self, n: int) -> bytes:
        if self._r.closed:
            raise IOError("socket关闭")
        return self._r.recv(n)

    def read_bytes(self, separator=b'\n') -> bytes:
        if self._r.closed:
            raise IOError("socket关闭")
        ba = bytearray()
        while True:
            b = self._r.recv(1)
            if(b==separator):
                break
            ba.append(int.from_bytes(b, byteorder="little"))
        return bytes(ba)

    def read_str(self, separator=b'\n') -> str:
        data = self.read_bytes(separator)
        if not data:
            return data.decode(encoding="utf-8")
        return None

    def write(self, p: bytes) -> int:
        if self._w.closed:
            raise IOError("socket关闭")
        length = len(p)
        if length > 0:
            self._w.send(p)
            return length
        return 0

    def write_str(self, s:str):
        if self._w.closed:
            raise IOError("socket关闭")
        if s != None:
            data = s.encode(encoding="utf-8")
            self._w.send(data)
            return len(data)
        return 0

    def flush(self):
        # self._w.close()
        # pass
        self._w.sendall("".encode(encoding="utf-8"))

def new_channel(sock) -> Channel:
    return Channel(id=Guid().string(), sock=sock)



from typing import List

from bee.db.psd.table import Table, to_table


class ColumnFilter():
    def filter(self, cols: List[str]) -> List[str]:
        pass


class Column():

    def __init__(self, name: str):
        pass

    def table(self) -> Table:
        pass

    def name(self) -> str:
        pass

    def alias(self) -> str:
        pass

    def field(self) -> str:
        pass


class Columns(List[Column]):
    def __init__(self, *items):
        if len(items) == 0:
            super().__init__([])
        else:
            super().__init__(items)

    @staticmethod
    def new_columns(*cols: str) -> "Columns":
        columns = Columns([])
        for c in cols:
            columns.append(SimpleColumn[SimpleColumn])
        return columns
    def simple(self, *cols: str) -> "Columns":
        columns = Columns()
        for col in cols:
            columns.append(SimpleColumn(col))
        return columns

    def get_table(self, t, *cols: str) -> "Columns":
        columns = Columns()
        for col in cols:
            columns.append(TableColumn(t=to_table(t),name=col))
        return columns

    def get_alias(self, t, col: str, alias: str) -> "Columns":
        columns = Columns()
        columns.append(TableColumn(t=to_table(t),name=col, alias=alias))
        return columns

    def get_expr(self, expr: str, alias: str) -> "Columns":
        columns = Columns()
        columns.append(ExprColumn(expr=expr, alias=alias))
        return columns

def new_columns(*cols: str) -> Columns:
    columns = Columns()
    for col in cols:
        # columns.append(SimpleColumn(col))
        if isinstance(col, str):
            columns.append(SimpleColumn(col))
        else:
            # TODO; complex logic, it's wrong
            columns.append(col)
    return columns


class SimpleColumn(str):

    def __init__(self, name: str):
        self._name = name
        super().__init__()

    def table(self) -> Table:
        return None

    def name(self) -> str:
        return self._name

    def alias(self) -> str:
        return ""

    def field(self) -> str:
        return self._name


def new_column(name: str, alias: str):
    if alias == None:
        SimpleColumn(name)
    else:
        new_table_column(name, alias)


class TableColumn(Column):
    name = "TableColumn"

    def __init__(self, t: Table, name: str, alias: str):
        self._table = t
        self._name = name
        self._alias = alias
        super.__init__()

    @staticmethod
    def new_table_column(t: Table, name: str, alias: str):
        tc = Table(t, name)
        if alias != None and alias != "":
            tc.alias = alias

        return tc

    def table(self) -> Table:
        return self._table

    def name(self) -> str:
        return self._name

    def alias(self) -> str:
        return self._alias

    def field(self) -> str:
        if self.alias == None or self.alias == "":
            return self.name
        else:
            return self.alias



def new_table_column(t: Table, name: str, alias: str):
    tc = Table(t, name)
    if alias != None and alias != "":
        tc.alias = alias

    return tc


class ExprColumn(Column):
    name = "ExprColumn"

    @staticmethod
    def new_expr_column(expr: str, alias: str) -> "ExprColumn":
        return ExprColumn(expr, alias)

    def __init__(self, expr: str, alias: str):
        self._expr = expr
        self._alias = alias
        super().__init__(expr)

    def table(self) -> Table:
        return None

    def name(self) -> str:
        return self._expr

    def expr(self) -> str:
        return self._expr

    def alias(self) -> str:
        return self._alias

    def field(self) -> str:
        return self._alias


def new_expr_column(expr: str, alias: str) -> ExprColumn:
    return ExprColumn(expr, alias)



from bee.data.guid.guid import Guid
from bee.net.rpc.registry.registry import Registry


class DirectRegistry(Registry):
    """Fake Registry"""

    def __init__(self, url: str):
        self.url = url

    def register(self, service: str, nid: str, address: str):
        pass

    def deregister(self, service: str, nid: str):
        pass

    def discovery(self, service):
        url = self.url
        if url:
            node = {Guid().string() : url}
            return node
        return {}
    def watch(self, service, callback):
        pass

    def close(self):
        pass


# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: msg.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='msg.proto',
  package='proto',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\tmsg.proto\x12\x05proto\"b\n\x07Request\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0f\n\x07service\x18\x02 \x01(\t\x12\x0e\n\x06method\x18\x03 \x01(\t\x12\x1c\n\x06labels\x18\x04 \x03(\x0b\x32\x0c.proto.Label\x12\x0c\n\x04\x61rgs\x18\x05 \x03(\x0c\"C\n\x08Response\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0e\n\x06result\x18\x02 \x01(\x0c\x12\x1b\n\x05\x65rror\x18\x03 \x01(\x0b\x32\x0c.proto.Error\"$\n\x05Label\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"6\n\x05\x45rror\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x0e\n\x06\x64\x65tail\x18\x03 \x01(\tb\x06proto3')
)




_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='proto.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='proto.Request.id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='service', full_name='proto.Request.service', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='method', full_name='proto.Request.method', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='proto.Request.labels', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='args', full_name='proto.Request.args', index=4,
      number=5, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=20,
  serialized_end=118,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='proto.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='proto.Response.id', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='result', full_name='proto.Response.result', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='proto.Response.error', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=187,
)


_LABEL = _descriptor.Descriptor(
  name='Label',
  full_name='proto.Label',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='proto.Label.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='proto.Label.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=189,
  serialized_end=225,
)


_ERROR = _descriptor.Descriptor(
  name='Error',
  full_name='proto.Error',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='code', full_name='proto.Error.code', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='proto.Error.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='detail', full_name='proto.Error.detail', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=227,
  serialized_end=281,
)

_REQUEST.fields_by_name['labels'].message_type = _LABEL
_RESPONSE.fields_by_name['error'].message_type = _ERROR
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['Label'] = _LABEL
DESCRIPTOR.message_types_by_name['Error'] = _ERROR
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'msg_pb2'
  # @@protoc_insertion_point(class_scope:proto.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'msg_pb2'
  # @@protoc_insertion_point(class_scope:proto.Response)
  ))
_sym_db.RegisterMessage(Response)

Label = _reflection.GeneratedProtocolMessageType('Label', (_message.Message,), dict(
  DESCRIPTOR = _LABEL,
  __module__ = 'msg_pb2'
  # @@protoc_insertion_point(class_scope:proto.Label)
  ))
_sym_db.RegisterMessage(Label)

Error = _reflection.GeneratedProtocolMessageType('Error', (_message.Message,), dict(
  DESCRIPTOR = _ERROR,
  __module__ = 'msg_pb2'
  # @@protoc_insertion_point(class_scope:proto.Error)
  ))
_sym_db.RegisterMessage(Error)


# @@protoc_insertion_point(module_scope)
